#### The multiple meanings of omnivory: empirical analysis, modular theory, and whole food web models
#### ------------------------------------------------------------------------------------------------
####
####  Anne M. McLeod, and Shawn J. Leroux
####
####
####  Code to replicate analyses for modular theory results
####  -------------------------------------------
####
####  CONTENTS:
####  ---------
####  1. Classic Omnivory module
####
####  2. Mutual Predation module
####
####  3. Multi-resource Omnivory module
####
####  4. Trophic Omnivory Module
#### 
####  5. Cannibalism

##### Packages
library(MASS)

#Assigning trophic levels
#install.packages(NetIndices)
library(NetIndices)

#Numerical Integration
#install.packages(deSolve)
library(deSolve)

#Compile Functions
#install.packages(compiler)
library(compiler)


## Part 1 - Classic Omnivory
## -------------------------------

#### Functions

# Function that calculates the change in resource biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, and Omnivore
R_eq <- function(state, parameters){
  r <- parameters$ri[1]
  R <- state[1]
  K <- parameters$K[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xo <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  O <- state[3]
  w21 <- parameters[1, 5]
  w31 <- parameters[1, 6]
  w32 <- parameters[2, 6]
  R_biomass <- r*R*(1-R/K) - (xc*y*C/e)*(w21*R/(0.5 + w21 * R)) - (xo*y*O/e)*(w31*R/(0.5 + w31 * R + w32 * C))
  return(R_biomass)
}
R_eq_compiled <- cmpfun(R_eq)

# Function that calculates the change in consumer biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, and Omnivore
C_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xo <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  O <- state[3]
  w21 <- parameters[1, 5]
  w31 <- parameters[1, 6]
  w32 <- parameters[2, 6]
  C_biomass <- -xc*C + xc*C*y*(w21*R/(0.5 + w21*R)) - (xo * y * O / e)*(w32 * C/ (0.5 + w31 * R + w32 * C))
  return(C_biomass)
}

C_eq_compiled <- cmpfun(C_eq)

# Function that calculates the change in omnivore biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, and Omnivore
O_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xo <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  O <- state[3]
  w21 <- parameters[1, 5]
  w31 <- parameters[1, 6]
  w32 <- parameters[2, 6]
  O_biomass <- -xo*O + (xo * y * O )*((w31 * R + w32 * C)/ (0.5 + w31 * R + w32 * C))
  return(O_biomass)
}

O_eq_compiled <- cmpfun(O_eq)

# Function that takes the current biomass of resource, consumer, and omnivore and calculates the change in biomass over 
# time. It is supplied to the function lsode in the package deSolve.
food_web <- function(t, state, parameters) {
  with(as.list(c(state, parameters)), {
    dS1 <- R_eq_compiled(state, parameters)
    dS2 <- C_eq_compiled(state, parameters)
    dS3 <- O_eq_compiled(state, parameters)
    list(c(dS1, dS2, dS3))
  })
}
food_web_compiled <- cmpfun(food_web)


# Function that takes the interaction matrix and the list of basal species, sets up the parameters for the model, calculates the change in biomass
# over 6000 time steps, and returns the time step that each species goes extinct (or 10 000 if it persists across time), the interaction strengths
# the min and max biomass of each species, and the initial resource, consumer, and omnivore biomass values.
calculation <- function(int_matrix, producer_list) {
  
  # Sets up the interaction strength matrix
  wij_matrix = matrix(nrow = S, ncol = S)
  wij_matrix[1, 2] <- 1
  wij_matrix[1, 3] <- runif(1,0,1)
  wij_matrix[2, 3] <- 1-wij_matrix[1,3]
  wij_matrix[is.na(wij_matrix)]<-0
  
  
  #Assigning body-mass based on a fixed predator-prey body-mass ratio as in Brose et al. 2006, where Mi = Z^P
  # where Z = 42 (Brose 2006) and P is the trophic position calculated as the prey averaged TP using NetIndices
  # packages
  
  
  TP <- TrophInd(int_matrix)
  tp_list <- TP[,1]
  
  
  bm_list <- vector(len = S)
  
  for (i in 1:S){
    if (producer_list[i] == 1){
      bm_list[i] <- 1
    }
    else {
      bm_list[i] <- 42^(tp_list[i]-1)
    }
  }
  
  
  # Sets up the parameter list supplied to the biomass functions
  colnames(int_matrix) <- c("Pred1", "Pred2", "Pred3")
  colnames(wij_matrix) <- c("w1", "w2", "w3")
  producer_list <- t(t(producer_list))
  colnames(producer_list) <- c("D")
  bm_list <- t(t(bm_list))
  colnames(bm_list) <- c("M")
  e <- matrix(0.85, nrow = S, ncol = 1)
  colnames(e) <- c("e")
  K <- matrix(1, nrow = S, ncol = 1)
  colnames(K) <- "K"
  Mb <- matrix(1, nrow = S, ncol = 1)
  colnames(Mb) <- "Mb"
  ar <- matrix(1, nrow = S, ncol = 1)
  colnames(ar) <- "ar"
  ax <- matrix(0.2227, nrow = S, ncol = 1)
  colnames(ax) <- "ax"
  ay <- matrix(1.7816, nrow = S, ncol = 1)
  colnames(ay) <- "ay"
  ri <- matrix(1, nrow = S, ncol = 1)
  colnames(ri) <- "ri"
  
  parameters <- as.data.frame(cbind(int_matrix,wij_matrix,producer_list, e, K, Mb, ar, ax, ay, ri, bm_list))
  
  #model is initializd with uniformly random popualtion denisities in terms of biomass densities (0.05 < Bi < 1)
  # and runs for t = 6000 time steps.
  t <- 6000
  R_init <- runif(1, 0.05, 1)
  C_init <- runif(1, 0.05, 1)
  O_init <- runif(1, 0.05, 1)
  state <- c(R_init, C_init, O_init)
  times <- seq(0, t, by = 1)
  
  
  out <- lsode(y = state, times = times, func = food_web_compiled, parms = parameters)
  
  # Determines the biomass minima and maxima for each species and the time each species goes extinct (or assigns a value of 10 000 if it persists)
  out2 <- as.data.frame(out)
  if (nrow(out2) > 6000){
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    
    persistence <- out2[which(out2$time > 5000),]
    
    Rmax <- ifelse(min(out2$`1`) < 10^-30, 0, max(persistence$`1`))
    Rmin <- ifelse(min(out2$`1`) < 10^-30, 0, min(persistence$`1`))
    Cmax <- ifelse(min(out2$`2`) < 10^-30, 0, max(persistence$`2`))
    Cmin <- ifelse(min(out2$`2`) < 10^-30, 0, min(persistence$`2`))
    Omax <- ifelse(min(out2$`3`) < 10^-30, 0, max(persistence$`3`))
    Omin <- ifelse(min(out2$`3`) < 10^-30, 0, min(persistence$`3`))
  } else{
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    
    persistence <- out2[which(out2$time > 5000),]
    
    Rmax <- (out2$`1`[nrow(out2)])
    Rmin <- (out2$`1`[nrow(out2)])
    Cmax <- (out2$`2`[nrow(out2)])
    Cmin <- (out2$`2`[nrow(out2)])
    Omax <- (out2$`3`[nrow(out2)])
    Omin <- (out2$`3`[nrow(out2)])
  }
  
  
  return(c(time1, time2, time3, wij_matrix[1, 1], wij_matrix[1, 2], wij_matrix[1, 3], wij_matrix[2, 1], wij_matrix[2, 2], wij_matrix[2, 3], wij_matrix[3, 1], wij_matrix[3, 2], wij_matrix[3, 3], Rmax, Rmin, Cmax, Cmin, Omax, Omin, R_init, C_init, O_init))
}

calculation_compiled <- cmpfun(calculation)


# The number of species
S <- 3

# Sets up the interaction matrix
int_matrix <- matrix(nrow = 3, ncol = 3)
int_matrix[1, 1] <- 0
int_matrix[1, 2] <- 1
int_matrix[1, 3] <- 1
int_matrix[2, 1] <- 0
int_matrix[2, 2] <- 0
int_matrix[2, 3] <- 1
int_matrix[3, 1] <- 0
int_matrix[3, 2] <- 0
int_matrix[3, 3] <- 0

# Determines which species are basal items
producer_list <- vector(len = S)
for (i in 1:ncol(int_matrix)){
  if (sum(int_matrix[,i]) == 0){
    producer_list[i] <- 1
  }
  else {
    producer_list[i] <- 0
  }
}

# Sets up the final matrix
final_Matrix <- matrix(nrow = 5000, ncol = 21)

# Runs through 5000 simulations
for (k in 1:5000){
  final_Matrix[k,] <- calculation_compiled(int_matrix, producer_list)
  print(k)
}


final_Matrix2 <- as.data.frame(final_Matrix)
names(final_Matrix2) <- c("t1", "t2", "t3", "w11", "w12", "w13", "w21", "w22", "w23", "w31", "w32", "w33", "Rmax", "Rmin", "Cmax", "Cmin", "Omax", "Omin", "R_init", "C_init", "O_init")

# Saves the dataframe
write.csv(final_Matrix2, "c_omnivory_results.csv")


## Part 2 - Mutual Predation
## -------------------------------

#### Functions

# Function that calculates the change in resource biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, and Omnivore
R_eq <- function(state, parameters){
  r <- parameters$ri[1]
  R <- state[1]
  K <- parameters$K[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xo <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  O <- state[3]
  w21 <- parameters[1, 5]
  w23 <- parameters[3, 5]
  w31 <- parameters[1, 6]
  w32 <- parameters[2, 6]
  R_biomass <- r*R*(1-R/K) - (xc*y*C/e)*(w21*R/(0.5 + w21 * R + w23 * O)) - (xo*y*O/e)*(w31*R/(0.5 + w31 * R + w32 * C))
  return(R_biomass)
}
R_eq_compiled <- cmpfun(R_eq)

# Function that calculates the change in consumer biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, and Omnivore
C_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xo <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  O <- state[3]
  w21 <- parameters[1, 5]
  w23 <- parameters[3, 5]
  w31 <- parameters[1, 6]
  w32 <- parameters[2, 6]
  C_biomass <- -xc*C + xc*C*y*((w21*R + w23 * O)/(0.5 + w21*R + w23 * O)) - (xo * y * O / e)*(w32 * C/ (0.5 + w31 * R + w32 * C))
  return(C_biomass)
}

C_eq_compiled <- cmpfun(C_eq)

# Function that calculates the change in omnivore biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, and Omnivore
O_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xo <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  O <- state[3]
  w21 <- parameters[1, 5]
  w23 <- parameters[3, 5]
  w31 <- parameters[1, 6]
  w32 <- parameters[2, 6]
  O_biomass <- -xo*O + (xo * y * O )*((w31 * R + w32 * C)/ (0.5 + w31 * R + w32 * C)) - (xc * y * C / e)*(w23 * O/ (0.5 + w21 * R + w23 * O))
  return(O_biomass)
}

O_eq_compiled <- cmpfun(O_eq)

# Function that takes the current biomass of resource, consumer, and omnivore and calculates the change in biomass over 
# time. It is supplied to the function lsode in the package deSolve.
food_web <- function(t, state, parameters) {
  with(as.list(c(state, parameters)), {
    dS1 <- R_eq_compiled(state, parameters)
    dS2 <- C_eq_compiled(state, parameters)
    dS3 <- O_eq_compiled(state, parameters)
    list(c(dS1, dS2, dS3))
  })
}
food_web_compiled <- cmpfun(food_web)


# Function that takes the interaction matrix and the list of basal species, sets up the parameters for the model, calculates the change in biomass
# over 6000 time steps, and returns the time step that each species goes extinct (or 10 000 if it persists across time), the interaction strengths
# the min and max biomass of each species, and the initial resource, consumer, and omnivore biomass values.
calculation <- function(int_matrix, producer_list) {
  
  
  wij_matrix = matrix(nrow = S, ncol = S)
  wij_matrix[1, 2] <- runif(1,0,1)
  wij_matrix[1, 3] <- runif(1,0,1)
  wij_matrix[2, 3] <- 1-wij_matrix[1,3]
  wij_matrix[3, 2] <- 1-wij_matrix[1,2]
  wij_matrix[is.na(wij_matrix)]<-0

  #Assigning body-mass based on a fixed predator-prey body-mass ratio as in Brose et al. 2006, where Mi = Z^P
  # where Z = 42 (Brose 2006) and P is the trophic position calculated as the prey averaged TP using NetIndices
  # packages
  
  
  TP <- TrophInd(int_matrix)
  tp_list <- TP[,1]
  
  
  bm_list <- vector(len = S)
  
  for (i in 1:S){
    if (producer_list[i] == 1){
      bm_list[i] <- 1
    }
    else {
      bm_list[i] <- 42^(tp_list[i]-1)
    }
  }
  
  
  
  colnames(int_matrix) <- c("Pred1", "Pred2", "Pred3")
  colnames(wij_matrix) <- c("w1", "w2", "w3")
  producer_list <- t(t(producer_list))
  colnames(producer_list) <- c("D")
  bm_list <- t(t(bm_list))
  colnames(bm_list) <- c("M")
  e <- matrix(0.85, nrow = S, ncol = 1)
  colnames(e) <- c("e")
  K <- matrix(1, nrow = S, ncol = 1)
  colnames(K) <- "K"
  Mb <- matrix(1, nrow = S, ncol = 1)
  colnames(Mb) <- "Mb"
  ar <- matrix(1, nrow = S, ncol = 1)
  colnames(ar) <- "ar"
  ax <- matrix(0.2227, nrow = S, ncol = 1)
  colnames(ax) <- "ax"
  ay <- matrix(1.7816, nrow = S, ncol = 1)
  colnames(ay) <- "ay"
  ri <- matrix(1, nrow = S, ncol = 1)
  colnames(ri) <- "ri"
  
  parameters <- as.data.frame(cbind(int_matrix,wij_matrix,producer_list, e, K, Mb, ar, ax, ay, ri, bm_list))
  
  #model is initializd with uniformly random popualtion denisities in terms of biomass densities (0.05 < Bi < 1)
  t <- 6000
  R_init <- runif(1, 0.05, 1)
  C_init <- runif(1, 0.05, 1)
  O_init <- runif(1, 0.05, 1)
  state <- c(R_init, C_init, O_init)
  times <- seq(0, t, by = 1)
  
  w21 = as.double(wij_matrix[1,2])
  w31 = as.double(wij_matrix[1,3])
  w32 = as.double(wij_matrix[2,3])
  w23 = as.double(wij_matrix[3,2])
  
  out <- lsode(y = state, times = times, func = food_web_compiled, parms = parameters)
  
  out2 <- as.data.frame(out)
  if (nrow(out2) > 6000){
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    
    persistence <- out2[which(out2$time > 5000),]
    Rmax <- ifelse(min(out2$`1`) < 10^-30, 0, max(persistence$`1`))
    Rmin <- ifelse(min(out2$`1`) < 10^-30, 0, min(persistence$`1`))
    Cmax <- ifelse(min(out2$`2`) < 10^-30, 0, max(persistence$`2`))
    Cmin <- ifelse(min(out2$`2`) < 10^-30, 0, min(persistence$`2`))
    Omax <- ifelse(min(out2$`3`) < 10^-30, 0, max(persistence$`3`))
    Omin <- ifelse(min(out2$`3`) < 10^-30, 0, min(persistence$`3`))
  } else {
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    
    persistence <- out2[which(out2$time > 5000),]
    
    Rmax <- (out2$`1`[nrow(out2)])
    Rmin <- (out2$`1`[nrow(out2)])
    Cmax <- (out2$`2`[nrow(out2)])
    Cmin <- (out2$`2`[nrow(out2)])
    Omax <- (out2$`3`[nrow(out2)])
    Omin <- (out2$`3`[nrow(out2)])
  }
  

  return(c(time1, time2, time3, wij_matrix[1, 1], wij_matrix[1, 2], wij_matrix[1, 3], wij_matrix[2, 1], wij_matrix[2, 2], wij_matrix[2, 3], wij_matrix[3, 1], wij_matrix[3, 2], wij_matrix[3, 3], Rmax, Rmin, Cmax, Cmin, Omax, Omin, R_init, C_init, O_init))
}


calculation_compiled <- cmpfun(calculation)


# The number of species
S <- 3

# Sets up the interaction matrix
int_matrix <- matrix(nrow = 3, ncol = 3)
int_matrix[1, 1] <- 0
int_matrix[1, 2] <- 1
int_matrix[1, 3] <- 1
int_matrix[2, 1] <- 0
int_matrix[2, 2] <- 0
int_matrix[2, 3] <- 1
int_matrix[3, 1] <- 0
int_matrix[3, 2] <- 1
int_matrix[3, 3] <- 0

# Determines which species are basal items
producer_list <- vector(len = S)
for (i in 1:ncol(int_matrix)){
  if (sum(int_matrix[,i]) == 0){
    producer_list[i] <- 1
  }
  else {
    producer_list[i] <- 0
  }
}

# Sets up the final matrix
final_Matrix <- matrix(nrow = 5000, ncol = 21)

# Runs through 5000 simulations
for (k in 1:5000){
  final_Matrix[k,] <- calculation_compiled(int_matrix, producer_list)
  
}

final_Matrix2 <- as.data.frame(final_Matrix)
names(final_Matrix2) <- c("t1", "t2", "t3", "w11", "w12", "w13", "w21", "w22", "w23", "w31", "w32", "w33", "Rmax", "Rmin", "Cmax", "Cmin", "Omax", "Omin", "R_init", "C_init", "O_init")

# Saves the dataframe
write.csv(final_Matrix2, "mutual_predation_results.csv")


## Part 3 - Multi-Resource Omnivory
## --------------------------------

#### Functions

# Function that calculates the change in the omnivore's resource biomass (R1) over time using the parameters supplied and the biomass of R1, R2,
# Consumer, and Omnivore
R1_eq <- function(state, parameters){
  r <- parameters$ri[1]
  R1 <- state[1]
  R2 <- state[2]
  K <- parameters$K[1]
  xc <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[3]
  O <- state[4]
  w41 <- parameters[1, S+4]
  w43 <- parameters[3, S+4]
  w32 <- parameters[2, S+3]
  R1_biomass <- r*R1*(1-R1/K) -  (xo*y*O/e)*(w41*R1/(0.5 + w41 * R1 + w43 * C))
  return(R1_biomass)
}
R1_eq_compiled <- cmpfun(R1_eq)

# Function that calculates the change in the consumer's resource biomass (R2) over time using the parameters supplied and the biomass of R1, R2,
# Consumer, and Omnivore
R2_eq <- function(state, parameters){
  r <- parameters$ri[1]
  R1 <- state[1]
  R2 <- state[2]
  K <- parameters$K[1]
  xc <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[3]
  O <- state[4]
  w41 <- parameters[1, S+4]
  w43 <- parameters[3, S+4]
  w32 <- parameters[2, S+3]
  R2_biomass <- r*R2*(1-R2/K) - (xc*y*C/e)*(w32*R2/(0.5 + w32 * R2)) 
  return(R2_biomass)
}
R2_eq_compiled <- cmpfun(R2_eq)


# Function that calculates the change in consumer biomass over time using the parameters supplied and the biomass of R1, R2,
# Consumer, and Omnivore
C_eq <- function(state, parameters){
  R1 <- state[1]
  R2 <- state[2]
  xc <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[3]
  O <- state[4]
  w41 <- parameters[1, S+4]
  w43 <- parameters[3, S+4]
  w32 <- parameters[2, S+3]
  C_biomass <- -xc*C + xc*C*y*(w32*R2/(0.5 + w32*R2)) - (xo * y * O / e)*(w43 * C/ (0.5 + w41 * R1 + w43 * C))
  return(C_biomass)
}

C_eq_compiled <- cmpfun(C_eq)


# Function that calculates the change in omnivore biomass over time using the parameters supplied and the biomass of R1, R2,
# Consumer, and Omnivore
O_eq <- function(state, parameters){
  R1 <- state[1]
  R2 <- state[2]
  xc <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[3]
  O <- state[4]
  w41 <- parameters[1, S+4]
  w43 <- parameters[3, S+4]
  w32 <- parameters[2, S+3]
  O_biomass <- -xo*O + (xo * y * O )*((w41 * R1 + w43 * C)/ (0.5 + w41 * R1 + w43 * C))
  return(O_biomass)
}

O_eq_compiled <- cmpfun(O_eq)

# Function that takes the current biomass of R1, R2, consumer, and omnivore and calculates the change in biomass over 
# time. It is supplied to the function lsode in the package deSolve.
food_web <- function(t, state, parameters) {
  with(as.list(c(state, parameters)), {
    dS1 <- R1_eq_compiled(state, parameters)
    dS2 <- R2_eq_compiled(state, parameters)
    dS3 <- C_eq_compiled(state, parameters)
    dS4 <- O_eq_compiled(state, parameters)
    list(c(dS1, dS2, dS3, dS4))
  })
}
food_web_compiled <- cmpfun(food_web)

# Function that takes the interaction matrix and the list of basal species, sets up the parameters for the model, calculates the change in biomass
# over 6000 time steps, and returns the time step that each species goes extinct (or 10 000 if it persists across time), the interaction strengths
# the min and max biomass of each species, and the initial resource, consumer, and omnivore biomass values.
calculation <- function(int_matrix, producer_list) {
  
  
  wij_matrix = matrix(nrow = S, ncol = S)
  wij_matrix[1, 4] <- runif(1,0,1)
  wij_matrix[2, 3] <- 1
  wij_matrix[3, 4] <- 1-wij_matrix[1,4]
  wij_matrix[is.na(wij_matrix)]<-0
  
  #Assigning body-mass based on a fixed predator-prey body-mass ratio as in Brose et al. 2006, where Mi = Z^P
  # where Z = 42 (Brose 2006) and P is the trophic position calculated as the prey averaged TP using NetIndices
  # packages
  
  
  TP <- TrophInd(int_matrix)
  tp_list <- TP[,1]
  
  
  bm_list <- vector(len = S)
  
  for (i in 1:S){
    if (producer_list[i] == 1){
      bm_list[i] <- 1
    }
    else {
      bm_list[i] <- 42^(tp_list[i]-1)
    }
  }
  
  
  
  colnames(int_matrix) <- c("Pred1", "Pred2", "Pred3", "Pred4")
  colnames(wij_matrix) <- c("w1", "w2", "w3", "w4")
  producer_list <- t(t(producer_list))
  colnames(producer_list) <- c("D")
  bm_list <- t(t(bm_list))
  colnames(bm_list) <- c("M")
  e <- matrix(0.85, nrow = S, ncol = 1)
  colnames(e) <- c("e")
  K <- matrix(1, nrow = S, ncol = 1)
  colnames(K) <- "K"
  Mb <- matrix(1, nrow = S, ncol = 1)
  colnames(Mb) <- "Mb"
  ar <- matrix(1, nrow = S, ncol = 1)
  colnames(ar) <- "ar"
  ax <- matrix(0.2227, nrow = S, ncol = 1)
  colnames(ax) <- "ax"
  ay <- matrix(1.7816, nrow = S, ncol = 1)
  colnames(ay) <- "ay"
  ri <- matrix(1, nrow = S, ncol = 1)
  colnames(ri) <- "ri"
  
  parameters <- as.data.frame(cbind(int_matrix,wij_matrix,producer_list, e, K, Mb, ar, ax, ay, ri, bm_list))
  
  
  #model is initializd with uniformly random popualtion denisities in terms of biomass densities (0.05 < Bi < 1)
  t <- 6000
  R1_init <- runif(1,0.05,1)
  R2_init <- runif(1,0.05,1)
  C_init <- runif(1,0.05,1)
  O_init <- runif(1,0.05,1)
  state <- c(R1_init, R2_init, C_init, O_init)
  times <- seq(0, t, by = 1)
  
  w41 = as.double(wij_matrix[1,4])
  w43 = as.double(wij_matrix[3,4])
  w32 = as.double(wij_matrix[2,3])
  
  out <- lsode(y = state, times = times, func = food_web_compiled, parms = parameters)
  
  out2 <- as.data.frame(out)
  if (nrow(out2) > 6000){
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    min4 <- ifelse(min(out2$`4`) < 10^-30, min(out2[which(out2$`4` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3, min4))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    time4 <- temp[4]
    
    persistence <- out2[which(out2$time > 5000),]

    R1max <- ifelse(min(out2$`1`) < 10^-30, 0, max(persistence$`1`))
    R1min <- ifelse(min(out2$`1`) < 10^-30, 0, min(persistence$`1`))
    R2max <- ifelse(min(out2$`2`) < 10^-30, 0, max(persistence$`2`))
    R2min <- ifelse(min(out2$`2`) < 10^-30, 0, min(persistence$`2`))
    Cmax <- ifelse(min(out2$`3`) < 10^-30, 0, max(persistence$`3`))
    Cmin <- ifelse(min(out2$`3`) < 10^-30, 0, min(persistence$`3`))
    Omax <- ifelse(min(out2$`4`) < 10^-30, 0, max(persistence$`4`))
    Omin <- ifelse(min(out2$`4`) < 10^-30, 0, min(persistence$`4`))
  } else {
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    min4 <- ifelse(min(out2$`4`) < 10^-30, min(out2[which(out2$`4` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3, min4))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    time4 <- temp[4]
    
    persistence <- out2[which(out2$time > 5000),]

    R1max <- (out2$`1`[nrow(out2)])
    R1min <- (out2$`1`[nrow(out2)])
    R2max <- (out2$`2`[nrow(out2)])
    R2min <- (out2$`2`[nrow(out2)])
    Cmax <- (out2$`3`[nrow(out2)])
    Cmin <- (out2$`3`[nrow(out2)])
    Omax <- (out2$`4`[nrow(out2)])
    Omin <- (out2$`4`[nrow(out2)])
  }
  
  return(c(time1, time2, time3, time4, wij_matrix[1, 1], wij_matrix[1, 2], wij_matrix[1, 3], wij_matrix[1,4], wij_matrix[2, 1], wij_matrix[2, 2], wij_matrix[2, 3], wij_matrix[2,4], wij_matrix[3, 1], wij_matrix[3, 2], wij_matrix[3, 3], wij_matrix[3,4], wij_matrix[4, 1], wij_matrix[4, 2], wij_matrix[4, 3], wij_matrix[4,4], R1max, R1min, R2max, R2min, Cmax, Cmin, Omax, Omin, R1_init, R2_init, C_init, O_init))
}

calculation_compiled <- cmpfun(calculation)

# The number of species
S <- 4

# Sets up the interaction matrix
int_matrix <- matrix(nrow = 4, ncol = 4)
int_matrix[1, 1] <- 0
int_matrix[1, 2] <- 0
int_matrix[1, 3] <- 0
int_matrix[1, 4] <- 1
int_matrix[2, 1] <- 0
int_matrix[2, 2] <- 0
int_matrix[2, 3] <- 1
int_matrix[2, 4] <- 0
int_matrix[3, 1] <- 0
int_matrix[3, 2] <- 0
int_matrix[3, 3] <- 0
int_matrix[3, 4] <- 1
int_matrix[4, 1] <- 0
int_matrix[4, 2] <- 0
int_matrix[4, 3] <- 0
int_matrix[4, 4] <- 0


# Determines which species are basal items
producer_list <- vector(len = S)
for (i in 1:ncol(int_matrix)){
  if (sum(int_matrix[,i]) == 0){
    producer_list[i] <- 1
  }
  else {
    producer_list[i] <- 0
  }
}

# Sets up the final matrix
final_Matrix <- matrix(nrow = 5000, ncol = 32)

# Runs through 5000 simulations
for (k in 1:5000){
  final_Matrix[k,] <- calculation_compiled(int_matrix, producer_list)
}


final_Matrix2 <- as.data.frame(final_Matrix)
names(final_Matrix2) <- c("t1", "t2", "t3", "t4", "w11", "w12", "w13", "w14", "w21", "w22", "w23", "w24", "w31", "w32", "w33", "w34", "w41", "w42", "w43", "w44", "R1max", "R1min", "R2max", "R2min", "Cmax", "Cmin", "Omax", "Omin", "R1_init", "R2_init", "C_init", "O_init")

# Saves the dataframe
write.csv(final_Matrix2, "multiresource_results.csv")

## Part 4 - Trophic Omnivory
## --------------------------------

#### Functions

# Function that calculates the change in the resource biomass over time using the parameters supplied and the biomass of resource, 
# Consumer, Predator, and Omnivore
R_eq <- function(state, parameters){
  r <- parameters$ri[1]
  R <- state[1]
  K <- parameters$K[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xp <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  P <- state[3]
  O <- state[4]
  w21 <- parameters[1, S+2]
  w32 <- parameters[2, S+3]
  w42 <- parameters[2, S+4]
  w43 <- parameters[3, S+4]
  R_biomass <- r*R*(1-R/K) -  (xc * C * y/e)*(w21 * R/(0.5 + w21 * R))
  return(R_biomass)
}
R_eq_compiled <- cmpfun(R_eq)

# Function that calculates the change in the consumer's biomass over time using the parameters supplied and the biomass of Resource, 
# Consumer, Predator, and Omnivore
C_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xp <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  P <- state[3]
  O <- state[4]
  w21 <- parameters[1, S+2]
  w32 <- parameters[2, S+3]
  w42 <- parameters[2, S+4]
  w43 <- parameters[3, S+4]
  C_biomass <- -xc * C + xc * C * y * (w21 * R/(0.5 + w21 * R)) - (xp * P * y)/e*(w32 * C/(0.5 + w32 * C)) - (xo * O * y/e) * (w42 * C)/(0.5 + w42 * C + w43 * P)
  return(C_biomass)
}
C_eq_compiled <- cmpfun(C_eq)


# Function that calculates the change in predator biomass over time using the parameters supplied and the biomass of Resouce,
# Consumer, Predator, and Omnivore
P_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xp <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  P <- state[3]
  O <- state[4]
  w21 <- parameters[1, S+2]
  w32 <- parameters[2, S+3]
  w42 <- parameters[2, S+4]
  w43 <- parameters[3, S+4]
  P_biomass <- -xp*P + xp*P*y*(w32*C/(0.5 + w32*C)) - (xo * y * O / e)*(w43 * P/ (0.5 + w43 * P + w42 * C))
  return(P_biomass)
}

P_eq_compiled <- cmpfun(P_eq)


# Function that calculates the change in omnivore biomass over time using the parameters supplied and the biomass of Resource,
# Consumer, Predator, and Omnivore
O_eq <- function(state, parameters){
  R <- state[1]
  xc <- (parameters$ax[2]/parameters$ar[2])*(parameters$M[2]/parameters$Mb[2])**(-1/4)
  xp <- (parameters$ax[3]/parameters$ar[3])*(parameters$M[3]/parameters$Mb[3])**(-1/4)
  xo <- (parameters$ax[4]/parameters$ar[4])*(parameters$M[4]/parameters$Mb[4])**(-1/4)
  y <- parameters$ay[1]/parameters$ax[1]
  e <- parameters$e[2]
  C <- state[2]
  P <- state[3]
  O <- state[4]
  w21 <- parameters[1, S+2]
  w32 <- parameters[2, S+3]
  w42 <- parameters[2, S+4]
  w43 <- parameters[3, S+4]
  O_biomass <- -xo*O + (xo * y * O )*((w42 * C + w43 * P)/ (0.5 + w42 * C + w43 * P))
  return(O_biomass)
}

O_eq_compiled <- cmpfun(O_eq)

# Function that takes the current biomass of Resource, Consumer, Predator, and Omnivore and calculates the change in biomass over 
# time. It is supplied to the function lsode in the package deSolve.
food_web <- function(t, state, parameters) {
  with(as.list(c(state, parameters)), {
    dS1 <- R_eq_compiled(state, parameters)
    dS2 <- C_eq_compiled(state, parameters)
    dS3 <- P_eq_compiled(state, parameters)
    dS4 <- O_eq_compiled(state, parameters)
    list(c(dS1, dS2, dS3, dS4))
  })
}
food_web_compiled <- cmpfun(food_web)

# Function that takes the interaction matrix and the list of basal species, sets up the parameters for the model, calculates the change in biomass
# over 6000 time steps, and returns the time step that each species goes extinct (or 10 000 if it persists across time), the interaction strengths
# the min and max biomass of each species, and the initial resource, consumer, and omnivore biomass values.
calculation <- function(int_matrix, producer_list) {
  
  wij_matrix = matrix(nrow = S, ncol = S)
  wij_matrix[1, 2] <- 1
  wij_matrix[2, 3] <- 1
  wij_matrix[2, 4] <- runif(1,0,1)
  wij_matrix[3, 4] <- 1 - wij_matrix[2,4]
  wij_matrix[is.na(wij_matrix)]<-0
  

  #Assigning body-mass based on a fixed predator-prey body-mass ratio as in Brose et al. 2006, where Mi = Z^P
  # where Z = 42 (Brose 2006) and P is the trophic position calculated as the prey averaged TP using NetIndices
  # packages
  
  
  TP <- TrophInd(int_matrix)
  tp_list <- TP[,1]
  
  
  bm_list <- vector(len = S)
  
  for (i in 1:S){
    if (producer_list[i] == 1){
      bm_list[i] <- 1
    }
    else {
      bm_list[i] <- 42^(tp_list[i]-1)
    }
  }
  
  
  
  colnames(int_matrix) <- c("Pred1", "Pred2", "Pred3", "Pred4")
  colnames(wij_matrix) <- c("w1", "w2", "w3", "w4")
  producer_list <- t(t(producer_list))
  colnames(producer_list) <- c("D")
  bm_list <- t(t(bm_list))
  colnames(bm_list) <- c("M")
  e <- matrix(0.85, nrow = S, ncol = 1)
  colnames(e) <- c("e")
  K <- matrix(1, nrow = S, ncol = 1)
  colnames(K) <- "K"
  Mb <- matrix(1, nrow = S, ncol = 1)
  colnames(Mb) <- "Mb"
  ar <- matrix(1, nrow = S, ncol = 1)
  colnames(ar) <- "ar"
  ax <- matrix(0.2227, nrow = S, ncol = 1)
  colnames(ax) <- "ax"
  ay <- matrix(1.7816, nrow = S, ncol = 1)
  colnames(ay) <- "ay"
  ri <- matrix(1, nrow = S, ncol = 1)
  colnames(ri) <- "ri"
  
  parameters <- as.data.frame(cbind(int_matrix,wij_matrix,producer_list, e, K, Mb, ar, ax, ay, ri, bm_list))
  
  #model is initializd with uniformly random popualtion denisities in terms of biomass densities (0.05 < Bi < 1)
  t <- 6000
  R_init <- runif(1, 0.05, 1)
  C_init <- runif(1, 0.05, 1)
  P_init <- runif(1, 0.05, 1)
  O_init <- runif(1, 0.05, 1)
  state <- c(R_init, C_init, P_init, O_init)
  times <- seq(0, t, by = 1)
  
  w21 <- as.double(wij_matrix[1,2])
  w32 <- as.double(wij_matrix[2,3])
  w42 <- as.double(wij_matrix[2,4])
  w43 <- as.double(wij_matrix[3,4])
  
  
  out <- lsode(y = state, times = times, func = food_web_compiled, parms = parameters)
  
  out2 <- as.data.frame(out)
  if (nrow(out2) > 6000){
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    min4 <- ifelse(min(out2$`4`) < 10^-30, min(out2[which(out2$`4` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3, min4))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    time4 <- temp[4]
    
    persistence <- out2[which(out2$time > 5000),]
    
    Rmax <- ifelse(min(out2$`1`) < 10^-30, 0, max(persistence$`1`))
    Rmin <- ifelse(min(out2$`1`) < 10^-30, 0, min(persistence$`1`))
    Cmax <- ifelse(min(out2$`2`) < 10^-30, 0, max(persistence$`2`))
    Cmin <- ifelse(min(out2$`2`) < 10^-30, 0, min(persistence$`2`))
    Pmax <- ifelse(min(out2$`3`) < 10^-30, 0, max(persistence$`3`))
    Pmin <- ifelse(min(out2$`3`) < 10^-30, 0, min(persistence$`3`))
    Omax <- ifelse(min(out2$`4`) < 10^-30, 0, max(persistence$`4`))
    Omin <- ifelse(min(out2$`4`) < 10^-30, 0, min(persistence$`4`))
  } else {
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    min3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
    min4 <- ifelse(min(out2$`4`) < 10^-30, min(out2[which(out2$`4` < 10^-30),]$time), 10000)
    temp <- sort(c(min1, min2, min3, min4))
    time1 <- temp[1]
    time2 <- temp[2]
    time3 <- temp[3]
    time4 <- temp[4]
    
    persistence <- out2[which(out2$time > 5000),]
    
    Rmax <- (out2$`1`[nrow(out2)])
    Rmin <- (out2$`1`[nrow(out2)])
    Cmax <- (out2$`2`[nrow(out2)])
    Cmin <- (out2$`2`[nrow(out2)])
    Pmax <- (out2$`3`[nrow(out2)])
    Pmin <- (out2$`3`[nrow(out2)])
    Omax <- (out2$`4`[nrow(out2)])
    Omin <- (out2$`4`[nrow(out2)])
  }
  
  
  return(c(time1, time2, time3, time4, wij_matrix[1, 1], wij_matrix[1, 2], wij_matrix[1, 3], wij_matrix[1,4], wij_matrix[2, 1], wij_matrix[2, 2], wij_matrix[2, 3], wij_matrix[2,4], wij_matrix[3, 1], wij_matrix[3, 2], wij_matrix[3, 3], wij_matrix[3,4], wij_matrix[4, 1], wij_matrix[4, 2], wij_matrix[4, 3], wij_matrix[4,4], Rmax, Rmin, Cmax, Cmin, Pmax, Pmin, Omax, Omin, R_init, C_init, P_init, O_init))
}


calculation_compiled <- cmpfun(calculation)


# The number of species
S <- 4

# Sets up the interaction matrix
int_matrix <- matrix(nrow = 4, ncol = 4)
int_matrix[1, 1] <- 0
int_matrix[1, 2] <- 1
int_matrix[1, 3] <- 0
int_matrix[1, 4] <- 0
int_matrix[2, 1] <- 0
int_matrix[2, 2] <- 0
int_matrix[2, 3] <- 1
int_matrix[2, 4] <- 1
int_matrix[3, 1] <- 0
int_matrix[3, 2] <- 0
int_matrix[3, 3] <- 0
int_matrix[3, 4] <- 1
int_matrix[4, 1] <- 0
int_matrix[4, 2] <- 0
int_matrix[4, 3] <- 0
int_matrix[4, 4] <- 0


# Determines which species are basal items
producer_list <- vector(len = S)
for (i in 1:ncol(int_matrix)){
  if (sum(int_matrix[,i]) == 0){
    producer_list[i] <- 1
  }
  else {
    producer_list[i] <- 0
  }
}

# Sets up the final matrix
final_Matrix <- matrix(nrow = 5000, ncol = 32)

# Runs through 5000 simulations
for (k in 1:5000){
  final_Matrix[k,] <- calculation_compiled(int_matrix, producer_list)
}

final_Matrix2 <- as.data.frame(final_Matrix)
names(final_Matrix2) <- c("t1", "t2", "t3", "t4", "w11", "w12", "w13", "w14", "w21", "w22", "w23", "w24", "w31", "w32", "w33", "w34", "w41", "w42", "w43", "w44", "Rmax", "Rmin", "Cmax", "Cmin", "Pmax", "Pmin", "Omax", "Omin", "R_init", "C_init", "P_init", "O_init")

# Saves the dataframe
write.csv(final_Matrix2, "t_omnivory_results.csv")


## Part 4 - Trophic Omnivory
## --------------------------------

#### Functions

# Function that calculates the change in the resource biomass over time using the parameters supplied and the biomass of resource and consumer
R_eq <- function(state, parameters){
  
  xc <- (parameters$ax[2]/parameters$ar[1])*(parameters$M[2]/parameters$Mb[1])**(-1/4)
  yc <- parameters$ay[1]/parameters$ax[1]
  R_biomass <- parameters$ri[1]*state[1]*(1-state[1]/parameters$K[1]) - (xc*yc*state[2]/parameters$e[2])*(parameters[1,4]*state[1]/(0.5 + parameters[1,4]*state[1] + parameters[2,4]*state[2]))
  return(R_biomass)
}
R_eq_compiled <- cmpfun(R_eq)

# Function that calculates the change in the consumer's biomass over time using the parameters supplied and the biomass of Resource and Consumer
C_eq <- function(state, parameters){
  xc <- (parameters$ax[2]/parameters$ar[1])*(parameters$M[2]/parameters$Mb[1])**(-1/4)
  yc <- parameters$ay[1]/parameters$ax[1]
  C_biomass <- - xc * state[2] + (xc*yc*state[2])*((parameters[1,4]*state[1] + parameters[2,4]*state[2])/(0.5 + parameters[1,4]*state[1] + parameters[2,4]*state[2])) - (xc*yc*state[2]/parameters$e[2])*(parameters[2,4]*state[2]/(0.5 + parameters[1,4]*state[1] + parameters[2,4]*state[2]))
  return(C_biomass)
}

C_eq_compiled <- cmpfun(C_eq)

# Function that takes the current biomass of Resource and Consumer and calculates the change in biomass over 
# time. It is supplied to the function lsode in the package deSolve.
food_web <- function(t, state, parameters) {
  with(as.list(c(state, parameters)), {
    dS1 <- R_eq_compiled(state, parameters)
    dS2 <- C_eq_compiled(state, parameters)
    list(c(dS1, dS2))
  })
}
food_web_compiled <- cmpfun(food_web)

# Function that takes the interaction matrix and the list of basal species, sets up the parameters for the model, calculates the change in biomass
# over 6000 time steps, and returns the time step that each species goes extinct (or 10 000 if it persists across time), the interaction strengths
# the min and max biomass of each species, and the initial resource, consumer, and omnivore biomass values.
calculation <- function(int_matrix, producer_list) {
  
  
  wij_matrix = matrix(nrow = S, ncol = S)
  wij_matrix[1, 2] <- runif(1,0,1)
  wij_matrix[2, 2] <- 1 - wij_matrix[1,2]
  
  wij_matrix[is.na(wij_matrix)]<-0
  
  #Assigning body-mass based on a fixed predator-prey body-mass ratio as in Brose et al. 2006, where Mi = Z^P
  # where Z = 42 (Brose 2006) and P is the trophic position calculated as the prey averaged TP using NetIndices
  # packages
  
  
  TP <- TrophInd(int_matrix)
  tp_list <- TP[,1]
  
  
  bm_list <- vector(len = S)
  
  for (i in 1:S){
    if (producer_list[i] == 1){
      bm_list[i] <- 1
    }
    else {
      bm_list[i] <- 42^(tp_list[i]-1)
    }
  }
  
  
  
  colnames(int_matrix) <- c("Pred1", "Pred2")
  colnames(wij_matrix) <- c("w1", "w2")
  producer_list <- t(t(producer_list))
  colnames(producer_list) <- c("D")
  bm_list <- t(t(bm_list))
  colnames(bm_list) <- c("M")
  e <- matrix(0.85, nrow = S, ncol = 1)
  colnames(e) <- c("e")
  K <- matrix(1, nrow = S, ncol = 1)
  colnames(K) <- "K"
  Mb <- matrix(1, nrow = S, ncol = 1)
  colnames(Mb) <- "Mb"
  ar <- matrix(1, nrow = S, ncol = 1)
  colnames(ar) <- "ar"
  ax <- matrix(0.2227, nrow = S, ncol = 1)
  colnames(ax) <- "ax"
  ay <- matrix(1.7816, nrow = S, ncol = 1)
  colnames(ay) <- "ay"
  ri <- matrix(1, nrow = S, ncol = 1)
  colnames(ri) <- "ri"
  
  parameters <- as.data.frame(cbind(int_matrix,wij_matrix,producer_list, e, K, Mb, ar, ax, ay, ri, bm_list))
  w21 <- wij_matrix[1,2]
  w22 <- wij_matrix[2,2]
  
  #state <- c(S1 = bm_list[1], S2 = bm_list[2], S3 = bm_list[3], S4 = bm_list[4], S5 = bm_list[5], S6 = bm_list[6], S7 = bm_list[7], S8 = bm_list[8], S9 = bm_list[9], S10 = bm_list[10])
  #state = bm_list
  #model is initializd with uniformly random popualtion denisities in terms of biomass densities (0.05 < Bi < 1)
  t <- 6000
  R_init <- runif(1, 0.05, 1)
  C_init <- runif(1, 0.05, 1)
  state <- c(R_init, C_init)
  times <- seq(0, t, by = 1)
  
  w21 <- as.double(wij_matrix[1,2])
  w22 <- as.double(wij_matrix[2,2])
  
  out <- lsode(y = state, times = times, func = food_web_compiled, parms = parameters)
  
  out2 <- as.data.frame(out)
  if (nrow(out2) > 6000){
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    temp <- min(c(min1, min2))
    
    persistence <- out2[which(out2$time > 5000),]

    Rmax <- ifelse(min(out2$`1`) < 10^-30, 0, max(persistence$`1`))
    Rmin <- ifelse(min(out2$`1`) < 10^-30, 0, min(persistence$`1`))
    Cmax <- ifelse(min(out2$`2`) < 10^-30, 0, max(persistence$`2`))
    Cmin <- ifelse(min(out2$`2`) < 10^-30, 0, min(persistence$`2`))
  } else {
    min1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
    min2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
    temp <- min(c(min1, min2))
    
    persistence <- out2[which(out2$time > 5000),]

    Rmax <- (out2$`1`[nrow(out2)])
    Rmin <- (out2$`1`[nrow(out2)])
    Cmax <- (out2$`2`[nrow(out2)])
    Cmin <- (out2$`2`[nrow(out2)])
  }
  
  return(c(min1, min2, wij_matrix[1, 1], wij_matrix[1, 2], wij_matrix[2, 1], wij_matrix[2, 2], Rmax, Rmin, Cmax, Cmin, R_init, C_init))
}

calculation_compiled <- cmpfun(calculation)


# The number of species
S <- 2

# Sets up the interaction matrix
int_matrix <- matrix(nrow = 2, ncol = 2)
int_matrix[1, 1] <- 0
int_matrix[1, 2] <- 1
int_matrix[2, 1] <- 0
int_matrix[2, 2] <- 1

# Determines which species are basal items
producer_list <- vector(len = S)
for (i in 1:ncol(int_matrix)){
  if (sum(int_matrix[,i]) == 0){
    producer_list[i] <- 1
  }
  else {
    producer_list[i] <- 0
  }
}

# Sets up the final matrix
final_Matrix <- matrix(nrow = 5000, ncol = 12)

# Runs through 5000 simulations
for (k in 1:5000){
  final_Matrix[k,] <- calculation_compiled(int_matrix, producer_list)
  print(k)
}

final_Matrix2 <- as.data.frame(final_Matrix)
names(final_Matrix2) <- c("time1", "time2", "R_Stability", "C_Stability", "w11", "w12", "w21", "w22", "Rmax", "Rmin", "Cmax", "Cmin", "eigen_1", "eigen_2", "R_init", "C_init")

# Saves the dataframe
write.csv(final_Matrix2, "cannibalism_results.csv")

