#### The multiple meanings of omnivory: empirical analysis, modular theory, and whole food web models
#### ------------------------------------------------------------------------------------------------
####
####  Anne M. McLeod, and Shawn J. Leroux
####
####
####  Code to replicate analyses for whole food web model results
####  -----------------------------------------------------------
####
####  CONTENTS:
####  ---------
####  1. Generating interaction and interaction strength matrices
####
####  2. Food web model
####
####  3. Calculating initial and final module counts, omnivore-resource interactions, and interaction strengths.
####
####  4. Statistical models and graphs


##### Packages
library(MASS)

#Assigning trophic levels
#install.packages(NetIndices)
library(NetIndices)

#Numerical Integration
#install.packages(deSolve)
library(deSolve)

#Compile Functions
#install.packages(compiler)
library(compiler)

#Graph factors influencing persistence
#install.packages(ggplot2)
library(ggplot2)

#Assemble multiple graphs on one page
#install.packages(gridExtra)
library(gridExtra)



## Part 1 - Generating interaction and interaction strength matrices
## -----------------------------------------------------------------

# Interaction matrices and interaction strength matrices were constructed first and then used for the numerical integration provided in  
#Part 2 of this code. This way the analyses code be repeated if the computing cluster failed or time limits were exceeded. Further, this
#meant that change in motif counts, changes in average motif interaction strengths, and changes in average overall interaction strengths
#could be calculated after the numerical integration was completed.

# Functions

# Niche Model takes the number of species (S) and the connectance (C) and returns an interaction matrix following the methods of 
# Williams & Martinez 2000
Niche_Model <- function(S, C){
  #Vectors for the species-specific niche values, radius of niche, and centre of niche
  N_value <- vector(len = S)
  r_value <- vector(len = S)
  c_value <- vector(len = S)
  
  
  #assigns a random niche value to each species: uniform random value [0, 1]
  for (i in 1:length(N_value)){
    N_value[i] <- runif(1, 0, 1)
  }
  
  #assigns a random radius value to each species: radius is assigned using a beta value to
  # randomly draw values from [0, 1] with shape1 parameter = 1, and shape2 parameter = 1/(2*C) - 1
  # so that the expected value is 2*C (as according to Williams and Martinez 2000)
  for (i in 1:length(r_value)){
    r <- rbeta(1, 1, 1/(2*C)-1)
    r_value[i] <- r*(N_value[i])
  }
  
  #assigns a random niche value to each species: uniform random value [ri/2, ni]
  for (i in 1:length(c_value)){
    c <- runif(1, r_value[i]/2, N_value[i])
    c_value[i] <- c
  }
  
  #Predator j consumes all prey i that are within its radius
  int_matrix <- matrix(nrow = S, ncol = S)
  for (j in 1:S){
    for (i in 1:S){
      if ((c_value[j] - r_value[j]/2) <= N_value[i] && N_value[i] <= (c_value[j] + r_value[j]/2)){
        int_matrix[i, j] <- 1
      }
      else {
        int_matrix[i, j] <- 0
      }
    }
  }
  return(int_matrix)
}
Niche_Model_compiled <- cmpfun(Niche_Model)


#interaction_Strengths takes the int_matrix formed using the Niche_Model function and assigns interaction strengths. 
# To do this we use a log-normal distribution with log-mean u = -3, and log-standard deviation sigma = 1.5 
interaction_strengths <- function(int_matrix){
  wij_matrix = matrix(nrow = S, ncol = S)
  for (i in 1:nrow(int_matrix)){
    for (j in 1:nrow(int_matrix)){
      if (int_matrix[i,j] == 1){
        wij_matrix[i, j] <- rlnorm(1, meanlog = -3, sdlog = 1.5)
        while(wij_matrix[i,j] > 1){
          wij_matrix[i, j] <- rlnorm(1, meanlog = -3, sdlog = 1.5)
        }
      }
      else {
        wij_matrix[i,j] <- 0
      }
    }
  }
  return(wij_matrix)
}
interaction_strengths_compiled <- cmpfun(interaction_strengths)

#set number of species (S) and connectance (C). For these analyses 50 species were always used, but connectance values were 0.1, 0.15, and 0.2
S <- 50
C <- 0.2

x <- 1
#For each set of connectance 250 different interaction matrices were created and 125 different interaction strength matrices of these
#250 different interaction matrices. This means there are 31250 interaction strength matrices for each value of connectance.
#These matrices were then saved according to their food web number (1-31250 for C = 0.1, 31251 - 62500 for C = 0.15, and 62501 - 93750 
#for C = 0.15)
for (k in 1:250){
  int_matrix <- Niche_Model_compiled(S, C)
  
  #Check to make sure that sum(row) never equals zero. If it does, then that species is not in the food web
  # and the food web needs to be redrawn
  i <- 1
  while (i <= nrow(int_matrix)){
    if (sum(int_matrix[i,]) == 0){
      int_matrix <- Niche_Model(S, C)
      i <- 1
    }
    else {
      i <- i + 1
    }
  }
  
  for (j in 1:125){
    wij_matrix <- interaction_strengths_compiled(int_matrix)
    wij_file_name <- paste("wij_matrix_", x, ".csv")
    int_file_name <- paste("int_matrix_", x, ".csv")
    write.csv(wij_matrix, wij_file_name)
    write.csv(int_matrix, int_file_name)
    x <- x + 1
  }
  
  print(k)
}



## Part 2 - Food web model
## -----------------------

#Functions

#This function takes the predator (i), the parameter values of all species (parmeters), and the biomass state of all species at that time (state)
#and calculates the denominator of the functional response.
functional_response <- function(i, parameters, state){
  ##NOTE: Just returns the bottom of the functional response.
  wij_temp <- parameters[,c((S+1):(2*S))]
  bottom <- sum(wij_temp[,i]*state)
  return(bottom)
}
functional_response_compiled <- cmpfun(functional_response)


#This function calculates the biomass of species (i) at the next time step using the biomass of all species at the current time step (state)
#and the species-specific parameters (parameters)
b_eq <- function(i, state, parameters){
  Gi <- 1 - state[i]/parameters$K[i]  #From Brose et al. 2006  #1-ifelse(i != 1 && i != 10, sum(state[1:i]) + sum(state[(i+1):10]), ifelse(i == 1, sum(state[(i+1):10]), sum(state[1:9])))/(parameters$K[i])
  xi <- (parameters$ax[i]/parameters$ar[i])*(parameters$M[i]/parameters$Mb[i])**(-1/4)
  yi <- parameters$ay[i]/parameters$ax[i]
  Fij <- 0
  bottom <- functional_response_compiled(i, parameters, state)
  if (length(bottom) > 0){
    Fij <- bottom/(0.5 + bottom)
  }
  ## Removing the loop from calculating the loss. 
  stateDF <- t((matrix(nrow = S, ncol = S, state)))
  wij_trans <- t((parameters[,c((S+1):(2*S))])) 
  Predation_Loss <- 0
  pred <- which(parameters[i, c(1:S)] > 0)
  if (length(pred) > 0){
    for (k in 1:length(pred)){
      xk <- (parameters$ax[pred[k]]/parameters$ar[pred[k]])*(parameters$M[pred[k]]/parameters$Mb[pred[k]])**(-1/4)
      yk <- parameters$ay[pred[k]]/parameters$ax[pred[k]]
      Bk <- state[pred[k]]
      Fki <- (parameters[i, S+pred[k]]*state[i])/(0.5 + functional_response_compiled(pred[k], parameters, state))
      eki <- parameters$e[pred[k]]
      Predation_Loss <- Predation_Loss + (xk*yk*Bk*Fki/eki)
    }
  }
  Biomass <- parameters$D[i]*parameters$ri[i]*Gi*state[i] + (parameters$D[i]-1)*state[i]*xi - (parameters$D[i]-1)*state[i]*xi*yi*Fij-state[i]*Predation_Loss #where parameters[i, 21] determines if it's a producer (1) or not

  return(Biomass)
}
b_eq_compiled <- cmpfun(b_eq)


#This function numerically integrates the b_eq function across 50 species, for a specied time (t), using the initial biomass of each of the
#50 species (state) and the given parameters for each species (parameters). This is the function called by the lsode function in deSolve
food_web <- function(t, state, parameters) {
  with(as.list(c(state, parameters)), {
    dS1 <- b_eq_compiled(1, state, parameters)
    dS2 <- b_eq_compiled(2, state, parameters)
    dS3 <- b_eq_compiled(3, state, parameters)
    dS4 <- b_eq_compiled(4, state, parameters)
    dS5 <- b_eq_compiled(5, state, parameters)
    dS6 <- b_eq_compiled(6, state, parameters)
    dS7 <- b_eq_compiled(7, state, parameters)
    dS8 <- b_eq_compiled(8, state, parameters)
    dS9 <- b_eq_compiled(9, state, parameters)
    dS10 <- b_eq_compiled(10, state, parameters)
    dS11 <- b_eq_compiled(11, state, parameters)
    dS12 <- b_eq_compiled(12, state, parameters)
    dS13 <- b_eq_compiled(13, state, parameters)
    dS14 <- b_eq_compiled(14, state, parameters)
    dS15 <- b_eq_compiled(15, state, parameters)
    dS16 <- b_eq_compiled(16, state, parameters)
    dS17 <- b_eq_compiled(17, state, parameters)
    dS18 <- b_eq_compiled(18, state, parameters)
    dS19 <- b_eq_compiled(19, state, parameters)
    dS20 <- b_eq_compiled(20, state, parameters)
    dS21 <- b_eq_compiled(21, state, parameters)
    dS22 <- b_eq_compiled(22, state, parameters)
    dS23 <- b_eq_compiled(23, state, parameters)
    dS24 <- b_eq_compiled(24, state, parameters)
    dS25 <- b_eq_compiled(25, state, parameters)
    dS26 <- b_eq_compiled(26, state, parameters)
    dS27 <- b_eq_compiled(27, state, parameters)
    dS28 <- b_eq_compiled(28, state, parameters)
    dS29 <- b_eq_compiled(29, state, parameters)
    dS30 <- b_eq_compiled(30, state, parameters)
    dS31 <- b_eq_compiled(31, state, parameters)
    dS32 <- b_eq_compiled(32, state, parameters)
    dS33 <- b_eq_compiled(33, state, parameters)
    dS34 <- b_eq_compiled(34, state, parameters)
    dS35 <- b_eq_compiled(35, state, parameters)
    dS36 <- b_eq_compiled(36, state, parameters)
    dS37 <- b_eq_compiled(37, state, parameters)
    dS38 <- b_eq_compiled(38, state, parameters)
    dS39 <- b_eq_compiled(39, state, parameters)
    dS40 <- b_eq_compiled(40, state, parameters)
    dS41 <- b_eq_compiled(41, state, parameters)
    dS42 <- b_eq_compiled(42, state, parameters)
    dS43 <- b_eq_compiled(43, state, parameters)
    dS44 <- b_eq_compiled(44, state, parameters)
    dS45 <- b_eq_compiled(45, state, parameters)
    dS46 <- b_eq_compiled(46, state, parameters)
    dS47 <- b_eq_compiled(47, state, parameters)
    dS48 <- b_eq_compiled(48, state, parameters)
    dS49 <- b_eq_compiled(49, state, parameters)
    dS50 <- b_eq_compiled(50, state, parameters)
    list(c(dS1, dS2, dS3, dS4, dS5, dS6, dS7, dS8, dS9, dS10, dS11, dS12, dS13, dS14, dS15, dS16, dS17, dS18, dS19, dS20, dS21, dS22, dS23, dS24, dS25, dS26, dS27, dS28, dS29, dS30, dS31, dS32, dS33, dS34, dS35, dS36, dS37, dS38, dS39, dS40, dS41, dS42, dS43, dS44, dS45, dS46, dS47, dS48, dS49, dS50))
  })
}
food_web_compiled <- cmpfun(food_web)


#This function sets up the parameter values using provided interaction matrices (int_matrix) and interaction strength matrices (wij_matrix).
#It randomly assigns species body-mass based on a fixed predator-prey body-mass ratio using trophic position (see Brose et al. 2006)
#Randomly assigns initial biomass values for each species, selects a length of time (4000 time steps) and returns overall persistence and
#the time step at which each species goes extinct (10000 if the species persists for all 4000 time steps)
calculation <- function(int_matrix, wij_matrix){
  
  
  #Determine if species is producer or not (for biomass eq'n calculations)
  producer_list <- vector(len = S)
  for (i in 1:ncol(int_matrix)){
    if (sum(int_matrix[,i]) == 0){
      producer_list[i] <- 1
    }
    else {
      producer_list[i] <- 0
    }
  }
  
  
  #Assigning body-mass based on a fixed predator-prey body-mass ratio as in Brose et al. 2006, where Mi = Z^P
  # where Z = 42 (Brose 2006) and P is the trophic position calculated as the prey averaged TP using NetIndices
  # packages
  
  
  TP <- TrophInd(int_matrix)
  tp_list <- TP[,1]
  
  
  bm_list <- vector(len = S)
  
  for (i in 1:S){
    if (producer_list[i] == 1){
      bm_list[i] <- 1
    }
    else {
      bm_list[i] <- 42^(tp_list[i]-1)
    }
  }
  
  
  
  colnames(int_matrix) <- c("Pred1", "Pred2", "Pred3", "Pred4", "Pred5", "Pred6", "Pred7", "Pred8", "Pred9", "Pred10", "Pred11", "Pred12", "Pred13", "Pred14", "Pred15", "Pred16", "Pred17", "Pred18", "Pred19", "Pred20", "Pred21", "Pred22", "Pred23", "Pred24", "Pred25", "Pred26", "Pred27", "Pred28", "Pred29", "Pred30", "Pred31", "Pred32", "Pred33", "Pred34", "Pred35", "Pred36", "Pred37", "Pred38", "Pred39", "Pred40", "Pred41", "Pred42", "Pred43", "Pred44", "Pred45", "Pred46", "Pred47", "Pred48", "Pred49", "Pred50")
  colnames(wij_matrix) <- c("w1", "w2", "w3", "w4", "w5", "w6", "w7", "w8", "w9", "w10", "w11", "w12", "w13", "w14", "w15", "w16", "w17", "w18", "w19", "w20", "w21", "w22", "w23", "w24", "w25", "w26", "w27", "w28", "w29", "w30", "w31", "w32", "w33", "w34", "w35", "w36", "w37", "w38", "w39", "w40", "w41", "w42", "w43", "w44", "w45", "w46", "w47", "w48", "w49", "w50")
  producer_list <- t(t(producer_list))
  colnames(producer_list) <- c("D")
  bm_list <- t(t(bm_list))
  colnames(bm_list) <- c("M")
  e <- matrix(0.85, nrow = S, ncol = 1)
  colnames(e) <- c("e")
  K <- matrix(1, nrow = S, ncol = 1)
  colnames(K) <- "K"
  Mb <- matrix(1, nrow = S, ncol = 1)
  colnames(Mb) <- "Mb"
  ar <- matrix(1, nrow = S, ncol = 1)
  colnames(ar) <- "ar"
  ax <- matrix(0.2227, nrow = S, ncol = 1)
  colnames(ax) <- "ax"
  ay <- matrix(1.7816, nrow = S, ncol = 1)
  colnames(ay) <- "ay"
  ri <- matrix(1, nrow = S, ncol = 1)
  colnames(ri) <- "ri"
  
  parameters <- as.data.frame(cbind(int_matrix,wij_matrix,producer_list, e, K, Mb, ar, ax, ay, ri, bm_list))
  
  
  #state <- c(S1 = bm_list[1], S2 = bm_list[2], S3 = bm_list[3], S4 = bm_list[4], S5 = bm_list[5], S6 = bm_list[6], S7 = bm_list[7], S8 = bm_list[8], S9 = bm_list[9], S10 = bm_list[10])
  #state = bm_list
  #model is initializd with uniformly random popualtion denisities in terms of biomass densities (0.05 < Bi < 1)
  t <- 6000
  state <- c(runif(50, 0.05,1))
  times <- seq(0, t, by = 1)
  
  
  out <- lsode(y = state, times = times, func = food_web_compiled, parms = parameters)
  
  #Calculating persistence = number of species with biomass > 10^-30 / S
  n_species <- S
  new_state <- out[nrow(out),c(2:51)]
  for (i in 1:length(new_state)){
    if (new_state[i] < 10^-30){
      n_species <- n_species - 1
      new_state[i] <- 0
    }
  }
  persistence <- n_species/S
  
  out2 <- as.data.frame(out)
  Sp1 <- ifelse(min(out2$`1`) < 10^-30, min(out2[which(out2$`1` < 10^-30),]$time), 10000)
  Sp2 <- ifelse(min(out2$`2`) < 10^-30, min(out2[which(out2$`2` < 10^-30),]$time), 10000)
  Sp3 <- ifelse(min(out2$`3`) < 10^-30, min(out2[which(out2$`3` < 10^-30),]$time), 10000)
  Sp4 <- ifelse(min(out2$`4`) < 10^-30, min(out2[which(out2$`4` < 10^-30),]$time), 10000)
  Sp5 <- ifelse(min(out2$`5`) < 10^-30, min(out2[which(out2$`5` < 10^-30),]$time), 10000)
  Sp6 <- ifelse(min(out2$`6`) < 10^-30, min(out2[which(out2$`6` < 10^-30),]$time), 10000)
  Sp7 <- ifelse(min(out2$`7`) < 10^-30, min(out2[which(out2$`7` < 10^-30),]$time), 10000)
  Sp8 <- ifelse(min(out2$`8`) < 10^-30, min(out2[which(out2$`8` < 10^-30),]$time), 10000)
  Sp9 <- ifelse(min(out2$`9`) < 10^-30, min(out2[which(out2$`9` < 10^-30),]$time), 10000)
  Sp10 <- ifelse(min(out2$`10`) < 10^-30, min(out2[which(out2$`10` < 10^-30),]$time), 10000)
  Sp11 <- ifelse(min(out2$`11`) < 10^-30, min(out2[which(out2$`11` < 10^-30),]$time), 10000)
  Sp12 <- ifelse(min(out2$`12`) < 10^-30, min(out2[which(out2$`12` < 10^-30),]$time), 10000)
  Sp13 <- ifelse(min(out2$`13`) < 10^-30, min(out2[which(out2$`13` < 10^-30),]$time), 10000)
  Sp14 <- ifelse(min(out2$`14`) < 10^-30, min(out2[which(out2$`14` < 10^-30),]$time), 10000)
  Sp15 <- ifelse(min(out2$`15`) < 10^-30, min(out2[which(out2$`15` < 10^-30),]$time), 10000)
  Sp16 <- ifelse(min(out2$`16`) < 10^-30, min(out2[which(out2$`16` < 10^-30),]$time), 10000)
  Sp17 <- ifelse(min(out2$`17`) < 10^-30, min(out2[which(out2$`17` < 10^-30),]$time), 10000)
  Sp18 <- ifelse(min(out2$`18`) < 10^-30, min(out2[which(out2$`18` < 10^-30),]$time), 10000)
  Sp19 <- ifelse(min(out2$`19`) < 10^-30, min(out2[which(out2$`19` < 10^-30),]$time), 10000)
  Sp20 <- ifelse(min(out2$`20`) < 10^-30, min(out2[which(out2$`20` < 10^-30),]$time), 10000)
  Sp21 <- ifelse(min(out2$`21`) < 10^-30, min(out2[which(out2$`21` < 10^-30),]$time), 10000)
  Sp22 <- ifelse(min(out2$`22`) < 10^-30, min(out2[which(out2$`22` < 10^-30),]$time), 10000)
  Sp23 <- ifelse(min(out2$`23`) < 10^-30, min(out2[which(out2$`23` < 10^-30),]$time), 10000)
  Sp24 <- ifelse(min(out2$`24`) < 10^-30, min(out2[which(out2$`24` < 10^-30),]$time), 10000)
  Sp25 <- ifelse(min(out2$`25`) < 10^-30, min(out2[which(out2$`25` < 10^-30),]$time), 10000)
  Sp26 <- ifelse(min(out2$`26`) < 10^-30, min(out2[which(out2$`26` < 10^-30),]$time), 10000)
  Sp27 <- ifelse(min(out2$`27`) < 10^-30, min(out2[which(out2$`27` < 10^-30),]$time), 10000)
  Sp28 <- ifelse(min(out2$`28`) < 10^-30, min(out2[which(out2$`28` < 10^-30),]$time), 10000)
  Sp29 <- ifelse(min(out2$`29`) < 10^-30, min(out2[which(out2$`29` < 10^-30),]$time), 10000)
  Sp30 <- ifelse(min(out2$`30`) < 10^-30, min(out2[which(out2$`30` < 10^-30),]$time), 10000)
  Sp31 <- ifelse(min(out2$`31`) < 10^-30, min(out2[which(out2$`31` < 10^-30),]$time), 10000)
  Sp32 <- ifelse(min(out2$`32`) < 10^-30, min(out2[which(out2$`32` < 10^-30),]$time), 10000)
  Sp33 <- ifelse(min(out2$`33`) < 10^-30, min(out2[which(out2$`33` < 10^-30),]$time), 10000)
  Sp34 <- ifelse(min(out2$`34`) < 10^-30, min(out2[which(out2$`34` < 10^-30),]$time), 10000)
  Sp35 <- ifelse(min(out2$`35`) < 10^-30, min(out2[which(out2$`35` < 10^-30),]$time), 10000)
  Sp36 <- ifelse(min(out2$`36`) < 10^-30, min(out2[which(out2$`36` < 10^-30),]$time), 10000)
  Sp37 <- ifelse(min(out2$`37`) < 10^-30, min(out2[which(out2$`37` < 10^-30),]$time), 10000)
  Sp38 <- ifelse(min(out2$`38`) < 10^-30, min(out2[which(out2$`38` < 10^-30),]$time), 10000)
  Sp39 <- ifelse(min(out2$`39`) < 10^-30, min(out2[which(out2$`39` < 10^-30),]$time), 10000)
  Sp40 <- ifelse(min(out2$`40`) < 10^-30, min(out2[which(out2$`40` < 10^-30),]$time), 10000)
  Sp41 <- ifelse(min(out2$`41`) < 10^-30, min(out2[which(out2$`41` < 10^-30),]$time), 10000)
  Sp42 <- ifelse(min(out2$`42`) < 10^-30, min(out2[which(out2$`42` < 10^-30),]$time), 10000)
  Sp43 <- ifelse(min(out2$`43`) < 10^-30, min(out2[which(out2$`43` < 10^-30),]$time), 10000)
  Sp44 <- ifelse(min(out2$`44`) < 10^-30, min(out2[which(out2$`44` < 10^-30),]$time), 10000)
  Sp45 <- ifelse(min(out2$`45`) < 10^-30, min(out2[which(out2$`45` < 10^-30),]$time), 10000)
  Sp46 <- ifelse(min(out2$`46`) < 10^-30, min(out2[which(out2$`46` < 10^-30),]$time), 10000)
  Sp47 <- ifelse(min(out2$`47`) < 10^-30, min(out2[which(out2$`47` < 10^-30),]$time), 10000)
  Sp48 <- ifelse(min(out2$`48`) < 10^-30, min(out2[which(out2$`48` < 10^-30),]$time), 10000)
  Sp49 <- ifelse(min(out2$`49`) < 10^-30, min(out2[which(out2$`49` < 10^-30),]$time), 10000)
  Sp50 <- ifelse(min(out2$`50`) < 10^-30, min(out2[which(out2$`50` < 10^-30),]$time), 10000)
  
  
  return(c(persistence, Sp1, Sp2, Sp3, Sp4, Sp5, Sp6, Sp7, Sp8, Sp9, Sp10, Sp11, Sp12, Sp13, Sp14, Sp15, Sp16, Sp17, Sp18, Sp19, Sp20, Sp21, Sp22, Sp23, Sp24, Sp25, Sp26, Sp27, Sp28, Sp29, Sp30,Sp31, Sp32, Sp33, Sp34, Sp35, Sp36, Sp37, Sp38, Sp39, Sp40, Sp41, Sp42, Sp43, Sp44, Sp45, Sp46, Sp47, Sp48, Sp49, Sp50))
}
calculation_compiled <- cmpfun(calculation)


#The Compute Canada Graham Computing Cluster was used to run these results so the following code takes the array index, opens the specified
#interaction matrix and interaction strength matrix, and then saves the result as a text file. In each text file are 20 different simulations
#lineNumber keeps track of which interaction and interaction strength matrix were used and serves as a Food Web Number index.
args_temp = commandArgs(trailingOnly=TRUE)
args = temp[as.integer(args_temp)]

# first argument is the line we want to start
start <- as.integer( args ) + 3999
startLine <- (start)*20 + 2000
# second argument is the number of lines we want to process
nLines    <- 20
lastLine  <- startLine + nLines - 1

S <- 50
C <- 0.1
final_Matrix <- matrix(nrow = nLines, ncol = 52)
x <- 1

# the loop going from startLine to lastLine
for (lineNumber in startLine:lastLine){
  print(paste("Processing line ", lineNumber))
  int_file <- paste("int_matrices/int_matrix_", lineNumber, ".csv")
  wij_file <- paste("wij_matrices/wij_matrix_", lineNumber, ".csv")
  int_matrix <- read.csv(int_file, header = TRUE)
  int_matrix <- int_matrix[,c(2:ncol(int_matrix))]
  wij_matrix <- read.csv(wij_file, header = TRUE)
  wij_matrix <- wij_matrix[,c(2:ncol(wij_matrix))]
 
  final_Matrix[x,] <- c(lineNumber, calculation_compiled(int_matrix, wij_matrix))
  x <- x + 1

}

filename <- paste(startLine, ".txt")
write.csv(final_Matrix, filename)


## Part 3 - Calculating initial and final module counts, omnivore-resource interactions, and interaction strengths.
## ----------------------------------------------------------------------------------------------------------------

#This code determines the initial number of each motif type, the final number of each motif type, the initial average interaction strength
#of each motif type, the final average interaction strength of each motif type, and the initial and final overall interaction strength.
#It requires the interaction and interaction strength matrices used in the food_web_model.R code, and the output file from the numerical
#integration provided in food_web_model.R code to determine which species do not persist and remove any interactions with those species for
#the final counts. Finally, it uses the food web number to link the results from the food_web_model.R code ith the initial interaction and 
#interaction strengnth matrices.


#### Packages
#Compiling functions
#install.packages("compiler")
library(compiler)


#### Functions

#This function counts the number of motifs, calculates the average interaction strength for each of these motifs, and calculates the average
#interaction strength for the whole interaction strength matrix. It requires an interaction matrix and an interaction strength matrix
#as inputs, and results in a list of the number of each motif found, the average interaction strength for each of these motifs, and then the
#overall average interaction strength.
motif_count <- function(int_matrix, wij_matrix){
  #Counting the motifs: "Classic Omnivory", "Multi-resource classic", "trophic omnivory", "mutual predation", "cannibalism
  cann_IS <- 0
  mp_IS <- 0
  c_omni_IS <- 0
  t_omni_IS <- 0
  multi_R_IS <- 0
  cannibalism <- 0
  mutual_predation <- 0
  c_omnivory <- 0
  t_omnivory <- 0
  multi_r_omnivory <- 0
  apparent_c <- 0
  exploitative_c <- 0
  tri_trophic_FC <- 0
  exploitative_IS <- 0
  apparent_IS <- 0
  tri_trophic_IS <- 0
  producer_list <- vector(len = 50)
  can_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  mp_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  t_omni_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  c_omni_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  mr_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  can_matrix_IS <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  mp_matrix_IS <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  t_omni_matrix_IS <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  c_omni_matrix_IS <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  mr_matrix_IS <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  for (i in 1:ncol(int_matrix)){
    if (sum(int_matrix[,i]) == 0){
      producer_list[i] <- 1
    }
    else {
      producer_list[i] <- 0
    }
  }
  
  for (r in 1:nrow(int_matrix)){
    for (c in 1:ncol(int_matrix)){
      if (int_matrix[r, c] == 1){
        for (x in 1:ncol(int_matrix)){
          if (int_matrix[x, c] == 1 && int_matrix[x, r] == 1 && c != x){
            if (r == c){
              cannibalism <- cannibalism + 1
              can_matrix[x,c] <- 1
              cann_IS <- cann_IS + wij_matrix[r,c] + wij_matrix[x,c]
              can_matrix_IS[x, c] <- wij_matrix[x,c]
              
            }
            else if (int_matrix[c,r] == 1){
              mutual_predation <- mutual_predation + 1
              mp_matrix[x, c] <- 1
              mp_matrix[x, r] <- 1
              mp_IS <- mp_IS + wij_matrix[c,r] + wij_matrix[r,c] + wij_matrix[x,c] + wij_matrix[x,r]
              mp_matrix_IS[x, c] <- wij_matrix[x,c]
              mp_matrix_IS[x, r] <- wij_matrix[x,r]
            }
            #it is either classic omnivory or trophic
            else if (producer_list[x] == 1){
              c_omnivory <- c_omnivory + 1
              c_omni_matrix[x, c] <- 1
              c_omni_IS <- c_omni_IS + wij_matrix[x,c] + wij_matrix[r,c] + wij_matrix[x,r]
              c_omni_matrix_IS[x,c] <- wij_matrix[x,c]
            }
            else {
              t_omnivory <- t_omnivory + 1
              t_omni_matrix[x, c] <- 1
              t_omni_IS <- t_omni_IS + wij_matrix[x,c] + wij_matrix[r,c] + wij_matrix[x,r]
              t_omni_matrix_IS[x,c] <- wij_matrix[x,c]
            }
          }
          else if (int_matrix[x, c] == 1 && producer_list[x] == 1){
            for (y in 1:nrow(int_matrix)){
              if (producer_list[y] == 1 && int_matrix[y, r] == 1 && y!=x){
                multi_r_omnivory <- multi_r_omnivory + 1
                mr_matrix[x, c] <- 1
                multi_R_IS <- multi_R_IS + wij_matrix[x, c] + wij_matrix[r, c] + wij_matrix[x, r]
                mr_matrix_IS[x,c] <- wij_matrix[x, c]
              }
            }
          }
          if (int_matrix[r,x] == 1 && x != c){
            exploitative_c <- exploitative_c + 1
          }
          if (int_matrix[x, c] == 1 && x != r){
            apparent_c <- apparent_c + 1
          }
          if (int_matrix[c, x] == 1 && x != c){
            tri_trophic_FC <- tri_trophic_FC + 1
          }
        }
      }
    }
  }
  can_matrix[is.na(can_matrix)] <- 0
  c_omni_matrix[is.na(c_omni_matrix)] <- 0
  t_omni_matrix[is.na(t_omni_matrix)] <- 0
  mp_matrix[is.na(mp_matrix)] <- 0
  mr_matrix[is.na(mr_matrix)] <- 0
  can_matrix_IS[is.na(can_matrix_IS)] <- 0
  c_omni_matrix_IS[is.na(c_omni_matrix_IS)] <- 0
  t_omni_matrix_IS[is.na(t_omni_matrix_IS)] <- 0
  mp_matrix_IS[is.na(mp_matrix_IS)] <- 0
  mr_matrix_IS[is.na(mr_matrix_IS)] <- 0
  mutual_predation <- ifelse(mutual_predation != 0, mutual_predation/2, 0)
  exploitative_c <- ifelse(exploitative_c != 0, exploitative_c/2, 0)
  apparent_c <- ifelse(apparent_c != 0, apparent_c/2, 0)
  
  wij_matrix <- mapply(wij_matrix, FUN = as.numeric)
  Average_IS <- mean(wij_matrix)
  
  return(c(cannibalism, mutual_predation, c_omnivory, t_omnivory, multi_r_omnivory, apparent_c, exploitative_c, tri_trophic_FC, cann_IS, mp_IS, c_omni_IS, t_omni_IS, multi_R_IS, exploitative_IS, apparent_IS, tri_trophic_IS, sum(can_matrix), sum(mp_matrix), sum(c_omni_matrix), sum(t_omni_matrix), sum(mr_matrix), sum(can_matrix_IS), sum(mp_matrix_IS), sum(c_omni_matrix_IS), sum(t_omni_matrix_IS), sum(mr_matrix_IS), Average_IS))
}
motif_compiled <- cmpfun(motif_count)

# Using the output from the numerical integration, need to create a final interaction matrix (& interaction strength matrix) where 
# all interactions with those species that do not persist become 0.

persist <- final_Matrix[,c(3:ncol(final_Matrix))]

# Save the module counts for both the final matrices and the initial matrices into two different data frames.
final_matrices <- matrix(nrow = nrow(final_Matrix), ncol = 28)
initial_matrices <- matrix(nrow = nrow(final_Matrix), ncol = 28)

y <- 1
for (x in 1:nrow(final_Matrix)){
  if (!(is.na(persist[x,1]))){
    int_file_name <- paste("int_matrices/int_matrix_", x , ".csv")
    int_matrix <- read.csv(int_file_name, header = TRUE)
    
    wij_file_name <- paste("wij_matrices/wij_matrix_", x , ".csv")
    wij_matrix <- read.csv(wij_file_name, header = TRUE)
    int_matrix <- int_matrix[,c(2:51)]
    wij_matrix <- wij_matrix[,c(2:51)]
    final_int_matrix <- int_matrix
    final_wij_matrix <- wij_matrix
    for (j in 1:50){
      if (persist[x,j] < 10000){
        final_int_matrix[j,] <- 0
        final_int_matrix[,j] <- 0
        final_wij_matrix[j,] <- 0
        final_wij_matrix[,j] <- 0
      }
    }
    initial_matrices[y,] <- c(x, motif_compiled(int_matrix, wij_matrix))
    final_matrices[y,] <- c(x, motif_compiled(final_int_matrix, final_wij_matrix))
    y <- y + 1
    print(y)
  }
}



## Part 4 - Statistical models and graphs
## ---------------------------------------

# Name the columns of the two matrices so that the column names are unique
names(initial_matrices) <- c("FW_Number", "cannibalism", "mutual_predation", "c_omnivory", "t_omnivory", "multi_r_omnivory", "apparent_c", "exploitative_c", "tri_trophic_FC", "cann_IS", "mp_IS", "c_omni_IS", "t_omni_IS", "multi_R_IS", "exploitative_IS", "apparent_IS", "tri_trophic_IS", "Cann_n", "MP_n", "C_omni_n", "T_omni_n", "MR_n", "Cann_int_IS", "MP_int_IS", "C_omni_int_IS", "T_omni_int_IS", "MR_int_IS", "Av_IS")
names(final_matrices) <- c("FW_Number", "final_cannibalism", "final_mutual_predation", "final_c_omnivory", "final_t_omnivory", "final_multi_r_omnivory", "final_apparent_c", "final_exploitative_c", "final_tri_trophic_FC", "final_cann_IS", "final_mp_IS", "final_c_omni_IS", "final_t_omni_IS", "final_multi_R_IS", "final_exploitative_IS", "final_apparent_IS", "final_tri_trophic_IS", "final_Cann_n", "final_MP_n", "final_C_omni_n", "final_T_omni_n", "final_MR_n", "final_Cann_int_IS", "final_MP_int_IS", "final_C_omni_int_IS", "final_T_omni_int_IS", "final_MR_int_IS", "Final_Av_IS")

#Combine the intial and final data frames into one
data <- merge(initial, final, by = "FW_Number")

#Calculate the average omnivore-resource interaction strength for each module type.
data$Cann_int_IS <- data$Cann_int_IS/data$Cann_n
data$MP_int_IS <- data$MP_int_IS/data$MP_n
data$C_omni_int_IS <- data$C_omni_int_IS/data$C_omni_n
data$T_omni_int_IS <- data$T_omni_int_IS/data$T_omni_n
data$MR_int_IS <- data$MR_int_IS/data$MR_n

data$final_Cann_int_IS <- ifelse(data$final_Cann_n > 0, data$final_Cann_int_IS/data$final_Cann_n, 0)
data$final_MP_int_IS <- ifelse(data$final_MP_n > 0, data$final_MP_int_IS/data$final_MP_n, 0)
data$final_C_omni_int_IS <- ifelse(data$final_C_omni_n >0 ,data$final_C_omni_int_IS/data$final_C_omni_n, 0)
data$final_T_omni_int_IS <- ifelse(data$final_T_omni_n > 0, data$final_T_omni_int_IS/data$final_T_omni_n, 0)
data$final_MR_int_IS <- ifelse(data$final_MR_n > 0, data$final_MR_int_IS/data$final_MR_n, 0)
data$Diff_IS <- data$Final_Av_IS - data$Av_IS

temp <- final_Matrix[,c(1,2)]
names(temp) <- c("FW_Number", "persistence")
Final_DF <- merge(temp, data, by = "FW_Number")

# Relationship between module frequency and persistence both statistical models and graphs

persistence_model <- lm(Final_DF$persistence~Final_DF$cannibalism+Final_DF$mutual_predation+Final_DF$c_omnivory+Final_DF$t_omnivory+Final_DF$multi_r_omnivory + Final_DF$apparent_c+Final_DF$exploitative_c+Final_DF$tri_trophic_FC)

partial_residuals <- resid(persistence_model, "partial")
partial_residuals <- cbind(partial_residuals, Final_DF[c(1:nrow(partial_residuals)),])
names(partial_residuals) <- c("p_cannibalism", "p_mp", "p_c_omnivory", "p_t_omnivory", "p_multi_r_omnivory", "p_apparent_c", "p_exploitative_c", "p_tri_trophic_FC", "FW_Number", "X", "X1", "Persistence", "X.x", names(partial_residuals)[14:92])

cor(partial_residuals$p_c_omnivory,partial_residuals$c_omnivory)
cor(partial_residuals$p_mp,partial_residuals$mutual_predation)
cor(partial_residuals$p_multi_r_omnivory,partial_residuals$multi_r_omnivory)
cor(partial_residuals$p_t_omnivory,partial_residuals$t_omnivory)
cor(partial_residuals$p_cannibalism,partial_residuals$cannibalism)

c_omni <- ggplot(partial_residuals, aes(x=c_omnivory, y=p_c_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

mp_omni <- ggplot(partial_residuals, aes(x=mutual_predation, y=p_mp) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

mr_omni <- ggplot(partial_residuals, aes(x=multi_r_omnivory, y=p_multi_r_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

t_omni <- ggplot(partial_residuals, aes(x=t_omnivory, y=p_t_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm",  colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

cannibalism <- ggplot(partial_residuals, aes(x=cannibalism, y=p_cannibalism) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", level = 0.95, colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))


plot_grid(c_omni, mp_omni, mr_omni, t_omni, cannibalism, ncol = 3)


##Classic Omnivory
summary(lm(partial_residuals$p_c_omnivory~partial_residuals$c_omnivory))

##Mutual Predation
summary(lm(partial_residuals$p_mp~partial_residuals$mutual_predation))

##Multi Resource Omnivory
summary(lm(partial_residuals$p_multi_r_omnivory~partial_residuals$multi_r_omnivory))

##Trophic Omnivory
summary(lm(partial_residuals$p_t_omnivory~partial_residuals$t_omnivory))


#cannibalism
summary(lm(partial_residuals$p_cannibalism~partial_residuals$cannibalism))


###########################
#Interaction frequency Without IS incorporated
###############################
library(ggplot2)
library(gridExtra)
library(cowplot)

persistence_model <- lm(Final_DF$persistence~Final_DF$Cann_n+Final_DF$MP_n+Final_DF$C_omni_n+Final_DF$T_omni_n+Final_DF$MR_n)

partial_residuals <- resid(persistence_model, "partial")
partial_residuals <- cbind(partial_residuals, Final_DF[c(1:nrow(partial_residuals)),])
names(partial_residuals) <- c("p_cannibalism", "p_mp", "p_c_omnivory", "p_t_omnivory", "p_multi_r_omnivory", "FW_Number", "X", "X1", "Persistence", "X.x", names(partial_residuals)[11:89])

cor(partial_residuals$p_c_omnivory,partial_residuals$C_omni_n)
cor(partial_residuals$p_mp,partial_residuals$MP_n)
cor(partial_residuals$p_multi_r_omnivory,partial_residuals$MR_n)
cor(partial_residuals$p_t_omnivory,partial_residuals$T_omni_n)
cor(partial_residuals$p_cannibalism,partial_residuals$C_omni_n)

c_omni <- ggplot(partial_residuals, aes(x=C_omni_n, y=p_c_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

mp_omni <- ggplot(partial_residuals, aes(x=MP_n, y=p_mp) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

mr_omni <- ggplot(partial_residuals, aes(x=MR_n, y=p_multi_r_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

t_omni <- ggplot(partial_residuals, aes(x=T_omni_n, y=p_t_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm",  colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

cannibalism <- ggplot(partial_residuals, aes(x=Cann_n, y=p_cannibalism) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", level = 0.95, colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))


plot_grid(c_omni, mp_omni, mr_omni, t_omni, cannibalism, ncol = 3)


##Classic Omnivory
summary(lm(partial_residuals$p_c_omnivory~partial_residuals$c_omnivory))

##Mutual Predation
summary(lm(partial_residuals$p_mp~partial_residuals$mutual_predation))

##Multi Resource Omnivory
summary(lm(partial_residuals$p_multi_r_omnivory~partial_residuals$multi_r_omnivory))

##Trophic Omnivory
summary(lm(partial_residuals$p_t_omnivory~partial_residuals$t_omnivory))


#cannibalism
summary(lm(partial_residuals$p_cannibalism~partial_residuals$cannibalism))


###########################
#Interaction frequency With IS incorporated
###############################
library(ggplot2)
library(gridExtra)
library(cowplot)

persistence_model <- lm(Final_DF$persistence~Final_DF$Cann_n+Final_DF$MP_n+Final_DF$C_omni_n+Final_DF$T_omni_n+Final_DF$MR_n+Final_DF$diff_c_omni_int_IS + Final_DF$diff_cann_int_IS + Final_DF$diff_mp_int_IS + Final_DF$diff_t_omni_int_IS+Final_DF$diff_mr_omni_int_IS)

partial_residuals <- resid(persistence_model, "partial")
partial_residuals <- cbind(partial_residuals, Final_DF[c(1:nrow(partial_residuals)),])
names(partial_residuals) <- c("p_cannibalism", "p_mp", "p_c_omnivory", "p_t_omnivory", "p_multi_r_omnivory", "p_diff_c_omni_int_IS" , "p_diff_cann_int_IS","p_diff_mp_int_IS", "p_diff_t_omni_int_IS", "p_diff_mr_omni_int_IS", "FW_Number", "X", "X1", "Persistence", "X.x", names(partial_residuals)[16:94])

cor(partial_residuals$p_c_omnivory,partial_residuals$C_omni_n)
cor(partial_residuals$p_mp,partial_residuals$MP_n)
cor(partial_residuals$p_multi_r_omnivory,partial_residuals$MR_n)
cor(partial_residuals$p_t_omnivory,partial_residuals$T_omni_n)
cor(partial_residuals$p_cannibalism,partial_residuals$C_omni_n)


c_omni <- ggplot(partial_residuals, aes(x=C_omni_n, y=p_c_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

mp_omni <- ggplot(partial_residuals, aes(x=MP_n, y=p_mp) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

mr_omni <- ggplot(partial_residuals, aes(x=MR_n, y=p_multi_r_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

t_omni <- ggplot(partial_residuals, aes(x=T_omni_n, y=p_t_omnivory) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm",  colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

cannibalism <- ggplot(partial_residuals, aes(x=Cann_n, y=p_cannibalism) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("")+
  geom_smooth(method= "lm", level = 0.95, colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))


plot_grid(c_omni, mp_omni, mr_omni, t_omni, cannibalism, ncol = 3)


##Classic Omnivory
summary(lm(partial_residuals$p_c_omnivory~partial_residuals$c_omnivory))

##Mutual Predation
summary(lm(partial_residuals$p_mp~partial_residuals$mutual_predation))

##Multi Resource Omnivory
summary(lm(partial_residuals$p_multi_r_omnivory~partial_residuals$multi_r_omnivory))

##Trophic Omnivory
summary(lm(partial_residuals$p_t_omnivory~partial_residuals$t_omnivory))


#cannibalism
summary(lm(partial_residuals$p_cannibalism~partial_residuals$cannibalism))


# Graph and statistical models for overall interaction strenghts


Init <- ggplot(Final_DF, aes(x=Av_IS, y=persistence) ) +
  geom_bin2d(bins = 70) +
  ylab("Persistence") + 
  xlab("Average Initial Interaction Strength")+
  geom_smooth(method= "lm", se = FALSE, colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

Final <- ggplot(Final_DF, aes(x=Final_Av_IS, y=persistence) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("Average Final Interaction Strength")+
  geom_smooth(method= "lm", se = FALSE, colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

Diff <- ggplot(Final_DF, aes(x=Diff_IS, y=persistence) ) +
  geom_bin2d(bins = 70) +
  ylab("") + 
  xlab("Change in Average Interaction Strengths")+
  geom_smooth(method= "lm", se = FALSE, colour = "black")+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))


plot_grid(Init, Final, Diff, ncol = 3)

cor(Final_DF$Initial_IS,Final_DF$persistence)
cor(Final_DF$Final_IS,Final_DF$persistence)
cor(Final_DF$Diff_IS, Final_DF$persistence)

summary(lm(Final_DF$Initial_IS~Final_DF$persistence))
summary(lm(Final_DF$Final_IS~Final_DF$persistence))
summary(lm(Final_DF$Diff_IS~Final_DF$persistence))

