#setwd("")

#### The multiple meanings of omnivory: empirical analysis, modular theory, and whole food web models
#### ------------------------------------------------------------------------------------------------
####
####  Anne M. McLeod, and Shawn J. Leroux
####
####
####  Code to replicate analyses for empirical module frequency and omnivore-resource interaction frequency results
####  -------------------------------------------
####
####  CONTENTS:
####  ---------
####  1. Observed number modules and interactions in the empirical web
####
####  2. Niche Model randomization
####
####  3. Local Rewiring Algorithm randomization
####
####  4. Nested Hierarchical I randomization
####
####  5. Nested Hierarchical II randomization
####
####  6. Sample probability calculation


##### Packages

#Assigning trophic levels
#install.packages(NetIndices)
library(NetIndices)

#Compile Functions
#install.packages(compiler)
library(compiler)

#Local Rewiring Algorithm using the package igraph
#install.packages(igraph)
library(igraph)


##### Functions

# Function that calculates the frequency of each type of omnivory module and each type of omnivore-resource interaction, it returns these counts
# in a list.
module_count <- function(int_matrix){
  #Counting the motifs: "Classic Omnivory", "Multi-resource classic", "trophic omnivory", "mutual predation", "cannibalism

  cannibalism <- 0
  mutual_predation <- 0
  c_omnivory <- 0
  t_omnivory <- 0
  multi_r_omnivory <- 0
  apparent_c <- 0
  exploitative_c <- 0
  tri_trophic_FC <- 0
  producer_list <- vector(len = nrow(int_matrix))
  can_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  mp_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  t_omni_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  c_omni_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  mr_matrix <- matrix(nrow = nrow(int_matrix), ncol = ncol(int_matrix))
  
  for (i in 1:ncol(int_matrix)){
    if (sum(int_matrix[,i]) == 0){
      producer_list[i] <- 1
    }
    else {
      producer_list[i] <- 0
    }
  }
  
  for (r in 1:nrow(int_matrix)){
    for (c in 1:ncol(int_matrix)){
      if (int_matrix[r, c] == 1){
        for (x in 1:ncol(int_matrix)){
          if (int_matrix[x, c] == 1 && int_matrix[x, r] == 1 && c != x){
            if (r == c){
              cannibalism <- cannibalism + 1
              can_matrix[x,c] <- 1
            }
            else if (int_matrix[c,r] == 1){
              mutual_predation <- mutual_predation + 1
              mp_matrix[x, c] <- 1
              mp_matrix[x, r] <- 1
            }
            #it is either classic omnivory or trophic
            else if (producer_list[x] == 1){
              c_omnivory <- c_omnivory + 1
              c_omni_matrix[x, c] <- 1
            }
            else {
              t_omnivory <- t_omnivory + 1
              t_omni_matrix[x, c] <- 1
            }
          }
          else if (int_matrix[x, c] == 1 && producer_list[x] == 1){
            for (y in 1:nrow(int_matrix)){
              if (producer_list[y] == 1 && int_matrix[y, r] == 1 && y!=x){
                multi_r_omnivory <- multi_r_omnivory + 1
                mr_matrix[x, c] <- 1
              }
            }
          }
          if (int_matrix[r,x] == 1 && x != c){
            exploitative_c <- exploitative_c + 1
          }
          if (int_matrix[x, c] == 1 && x != r){
            apparent_c <- apparent_c + 1
          }
          if (int_matrix[c, x] == 1 && x != c){
            tri_trophic_FC <- tri_trophic_FC + 1
          }
        }
      }
    }
  }
  can_matrix[is.na(can_matrix)] <- 0
  c_omni_matrix[is.na(c_omni_matrix)] <- 0
  t_omni_matrix[is.na(t_omni_matrix)] <- 0
  mp_matrix[is.na(mp_matrix)] <- 0
  mr_matrix[is.na(mr_matrix)] <- 0
  mutual_predation <- ifelse(mutual_predation != 0, mutual_predation/2, 0)
  exploitative_c <- ifelse(exploitative_c != 0, exploitative_c/2, 0)
  apparent_c <- ifelse(apparent_c != 0, apparent_c/2, 0)
  return(c(cannibalism, mutual_predation, c_omnivory, t_omnivory, multi_r_omnivory, apparent_c, exploitative_c, tri_trophic_FC, sum(can_matrix), sum(mp_matrix), sum(c_omni_matrix), sum(t_omni_matrix), sum(mr_matrix)))
  
}
module_count_compiled <- cmpfun(module_count)

## Part 2. Niche Model Randomization

Niche_Model <- function(S, C){
  #Vectors for the species-specific niche values, radius of niche, and centre of niche
  N_value <- vector(len = S)
  r_value <- vector(len = S)
  c_value <- vector(len = S)
  
  
  #assigns a random niche value to each species: uniform random value [0, 1]
  for (i in 1:length(N_value)){
    N_value[i] <- runif(1, 0, 1)
  }
  
  #assigns a random radius value to each species: radius is assigned using a beta value to
  # randomly draw values from [0, 1] with shape1 parameter = 1, and shape2 parameter = 1/(2*C) - 1
  # so that the expected value is 2*C (as according to Williams and Martinez 2000)
  for (i in 1:length(r_value)){
    r <- rbeta(1, 1, 1/(2*C)-1)
    r_value[i] <- r*(N_value[i])
  }
  
  #assigns a random niche value to each species: uniform random value [ri/2, ni]
  for (i in 1:length(c_value)){
    c <- runif(1, r_value[i]/2, N_value[i])
    c_value[i] <- c
  }
  
  #Predator j consumes all prey i that are within its radius
  int_matrix <- matrix(nrow = S, ncol = S)
  for (j in 1:S){
    for (i in 1:S){
      if ((c_value[j] - r_value[j]/2) <= N_value[i] && N_value[i] <= (c_value[j] + r_value[j]/2)){
        int_matrix[i, j] <- 1
      }
      else {
        int_matrix[i, j] <- 0
      }
    }
  }
  return(int_matrix)
}
niche_model_compiled <- cmpfun(Niche_Model)

# Part 4 - Nested Hierarchical I Randomization

# Takes the empirical fod web and assigns links based on the Nested Hierarchical I model presented Bascompte and Melian (2005), returns
# the new food web
Nested_Hierarchical_1 <- function(observed_matrix){
  #NOTE: in the Bascompte and Melian version of this (and Cattin version) they say "Starting with the smallest consumers'
  # niche value, the trophic links are attributed to consumers in a two-stage process. In stage one, prey species of consumer
  # j are randomly chosen among species with rank < j. Depending on this randomly chosen prey i, two cases are possible
  # (1) prey i has no consumer and therefore the next prey of consumer j will again be randomly attributed (With rank of prey < j)
  # (2) prey i already has one or more consumers and therefore consumer j joins the group of species is consumers, and the next
  # prey of consumer j is then randomly chosen among the set of prey of this group. ** However, if the number of prey in the group is
  #too small for choosing all remaining prey of consumer j, the remaining prey are again randomly chosen among prey wihtout 
  # consumers (with rank < j)
  # Also, in the Bascompte and Melian version they say they use the niche model, but they rank them based on the niches, so
  # probably actually use the cascade model. Here I did use the niche model (relaxing the connectance for pickign the radii
  # of the niches), so that I could still observe mutual predation and cannibalism which aren't permitted in the cascade model.
  # BUT relaxed the bottom of the niche range. for similar reasons.
  S = nrow(observed_matrix)
  C = 0.5   #Can't do perfect connectance because of the beta distribution, so this is the highest can do. So that the
  # model assigns links based on # of consumer interactions, not on connectance (As in the Niche Model)
  #Vectors for the species-specific niche values, radius of niche, and centre of niche
  N_value <- vector(len = S)
  r_value <- vector(len = S)
  c_value <- vector(len = S)
  
  
  #assigns a random niche value to each species: uniform random value [0, 1]
  for (i in 1:length(N_value)){
    N_value[i] <- runif(1, 0, 1)
  }
  N_value <- sort(N_value)
  
  #assigns a random radius value to each species: radius is assigned using a beta value to
  # randomly draw values from [0, 1] with shape1 parameter = 1, and shape2 parameter = 1/(2*C) - 1
  # so that the expected value is 2*C (as according to Williams and Martinez 2000)
  for (i in 1:length(r_value)){
    r <- rbeta(1, 1, 1/(2*C)-1)
    r_value[i] <- r*(N_value[i])
  }
  
  #assigns a random niche value to each species: uniform random value [ri/2, ni]
  for (i in 1:length(c_value)){
    c <- runif(1, r_value[i]/2, N_value[i])
    c_value[i] <- c
  }
  
  
  observed_matrix <- observed_matrix[,order(colSums(observed_matrix))]
  
  #Predator j consumes all prey i that are within its radius
  NewM <- matrix(nrow = S, ncol = S)
  NewM[is.na(NewM)] <- 0
  for (j in 1:S){
    no_prey <- colSums(observed_matrix)[j]
    i <- 0
    k <- round(runif(1,1,S))
    while (i != no_prey){
      
      if (N_value[k] <= (c_value[j] + r_value[j]/2) && NewM[k, j] == 0){
        if (rowSums(NewM)[k] == 0){
          NewM[k, j] <- 1
          i <- i + 1
        }
        else{
          prey <- vector(len = S)
          for (m in 1:j){
            
            prey_temp <- vector(len = S)
            if (NewM[k,m] != 0){
              
              temp <- 1
              for (r in 1:S){
                if (NewM[r,m]){
                  prey_temp[temp] <- r 
                  temp <- temp + 1
                }
              } 
            }
            prey <- union(prey, prey_temp)
          }
          prey <- prey[which(prey > 0)]
          if (length(prey) >0){
            NewM[k, j] <- 1
            i <- i + 1
            n <- 1
            while (i != no_prey && n < length(prey)){
              k <- round(runif(1, 1, length(prey)))
              if (NewM[prey[k], j] == 0){
                NewM[prey[k], j] <- 1
                i <- i + 1
              }
              n <- n + 1
            }
            if (i != no_prey){
              k <- round(runif(1, 1, length(prey)))
              if (N_value[k] <= (c_value[j] + r_value[j]/2) && NewM[k, j] == 0){
                if (rowSums(NewM)[k] == 0){
                  NewM[k, j] <- 1
                  i <- i + 1
                }
              }
            }
          }          
        }
      }
      k <- round(runif(1,1,S))
    }
  }
  return(NewM)
}

Nested_Hierarchical_1_compiled <- cmpfun(Nested_Hierarchical_1)

# Part 5 - Nested Hierarchical II Model randomization

# Functoin that takes the empirical food web and returns a randomization of it based on the Nested Hierarchical II model presented in Bascompte
# and Melian (2005)
Nested_Hierarchy_2 <- function(observed_matrix){
  S = nrow(observed_matrix)
  C = 0.5   #Can't do perfect connectance because of the beta distribution, so this is the highest can do. So that the
  # model assigns links based on # of consumer interactions, not on connectance (As in the Niche Model)
  #Vectors for the species-specific niche values, radius of niche, and centre of niche
  N_value <- vector(len = S)
  r_value <- vector(len = S)
  c_value <- vector(len = S)
  
  
  #assigns a random niche value to each species: uniform random value [0, 1]
  for (i in 1:length(N_value)){
    N_value[i] <- runif(1, 0, 1)
  }
  N_value <- sort(N_value)
  
  #assigns a random radius value to each species: radius is assigned using a beta value to
  # randomly draw values from [0, 1] with shape1 parameter = 1, and shape2 parameter = 1/(2*C) - 1
  # so that the expected value is 2*C (as according to Williams and Martinez 2000)
  for (i in 1:length(r_value)){
    r <- rbeta(1, 1, 1/(2*C)-1)
    r_value[i] <- r*(N_value[i])
  }
  
  #assigns a random niche value to each species: uniform random value [ri/2, ni]
  for (i in 1:length(c_value)){
    c <- runif(1, r_value[i]/2, N_value[i])
    c_value[i] <- c
  }
  
  
  observed_matrix <- observed_matrix[,order(colSums(observed_matrix))]
  
  #Predator j consumes all prey i that are within its radius
  NewM <- matrix(nrow = S, ncol = S)
  NewM[is.na(NewM)] <- 0
  for (j in 1:S){
    no_prey <- colSums(observed_matrix)[j]
    i <- 0
    k <- round(runif(1,1,S))
    while (i != no_prey){
      
      if (N_value[k] <= (c_value[j] + r_value[j]/2) && NewM[k, j] == 0){
        if (rowSums(NewM)[k] == 0){
          NewM[k, j] <- 1
          i <- i + 1
        }
        else{
          prey <- vector(len = S)
          for (m in 1:j){
            
            prey_temp <- vector(len = S)
            if (NewM[k,m] != 0){
              
              temp <- 1
              for (r in 1:S){
                if (NewM[r,m]){
                  prey_temp[temp] <- r 
                  temp <- temp + 1
                }
              } 
            }
            prey <- union(prey, prey_temp)
          }
          prey <- prey[which(prey > 0)]
          if (length(prey) >0){
            NewM[k, j] <- 1
            i <- i + 1
            n <- 1
            while (i != no_prey && n < length(prey)){
              k <- round(runif(1, 1, length(prey)))
              if (NewM[prey[k], j] == 0){
                NewM[prey[k], j] <- 1
                i <- i + 1
              }
              n <- n + 1
            }
            if (i != no_prey){
              pred <- vector(len = S)
              temp <- 1
              for (m in 1:j){
                
                
                if (NewM[k,m] != 0){
                  pred[temp] <- m
                  temp <- temp + 1
                }
              }
              pred <- pred[which(pred > 0)]
              if (length(pred) > 0){
                n <- 1
                while (i != no_prey && n < length(pred)){
                  k <- round(runif(1, 1, length(pred)))
                  if (NewM[pred[k], j] == 0){
                    NewM[pred[k], j] <- 1
                    i <- i + 1
                  }
                  n <- n + 1
                }
                
              }
              if (i != no_prey){
                k <- round(runif(1, 1, length(prey)))
                if (N_value[k] <= (c_value[j] + r_value[j]/2) && NewM[k, j] == 0){
                  if (rowSums(NewM)[k] == 0){
                    NewM[k, j] <- 1
                    i <- i + 1
                  }
                }
              }
            }
          }
          
        }
      }
      k <- round(runif(1,1,S))
    }
  }
  return(NewM)
}

Nested_Hierarchy_2_Compiled <- cmpfun(Nested_Hierarchy_2)


## Part 1 - Observed number modules and interactions in the empirical web
## ----------------------------------------------------------------------

# Load empirical food web in column format with two columns for predator and prey
# e.g. Silwood
Silwood <- read.csv("C:/Users/Anne/Dropbox/PhD/Omnivory Chapter/silwood.csv", header = TRUE)

# Create a list of all species in in the empirical web
Species <- union(unique(Silwood$Predator),unique(Silwood$Prey))

# Set up the column list of interactions into a matrix with predators as columns and prey as rows. A 1 indicates an interaction
int_matrix <- matrix(nrow = length(Species), ncol = length(Species))
colnames(int_matrix) <- Species
row.names(int_matrix) <- Species

for (j in 1:length(unique(Silwood$Predator))){
  Pred <- as.character(unique(Silwood$Predator)[j])
  Pred_sub <- subset(Silwood, Silwood$Predator == Pred)
  for (i in 1:nrow(Pred_sub)){
    Prey <- as.character(Pred_sub$Prey[i])
    int_matrix[Prey,Pred] <- 1
  }
}
int_matrix[is.na(int_matrix)] <- 0

# Count the number of modules and omnivore-resource interactions present in the empirical food web (in this case Silwood)
actual <- module_count_compiled(int_matrix)

Observed_Modules <- data.frame(double(), double(), double(), double(), double(), double(), double(), double(), double(), double(), double(), double(), double())
names(Observed_Modules) <- c("Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link")
Observed_Modules <- rbind(Observed_Modules, actual)
Observed_Modules$Foodweb <- ""
Observed_Modules$Foodweb[1] <- "Benguela"



## Part 2 - Niche Model Randomization
## ----------------------------------------------------------------------

#Count the number of modules and omnivore-resource interactions observed in 5000 randomizations using the Niche Model
S <- length(Species)
C <- sum(int_matrix)/(S^2)

random_modules <- matrix(nrow = 5000, ncol = 14)
names(random_modules) <- c("Iteration", "Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link")
for (i in 1:5000){
  print(i)
  int_matrix <- niche_model_compiled(S, C)
  random_modules[i,] <- c(i, module_count_compiled(int_matrix))
  
}
random_modules <- as.data.frame(random_modules)
write.csv(random_modules, "Benguela_Niche.csv")


## Part 3 - Local Rewiring Algorithm Randomizations
## ----------------------------------------------------------------------

Silwood_int <- int_matrix
random_modules <- matrix(nrow = 5000, ncol = 14)
names(random_modules) <- c("Iteration", "Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link")
g <- graph_from_adjacency_matrix(Silwood_int)

for (k in 1:5000) {
  newM <- matrix(nrow = nrow(Silwood_int), ncol = ncol(Silwood_int))
  g2 <- rewire(g, with = keeping_degseq(loops = TRUE, niter = sum(Silwood_int)))
  g3 <- as_adjacency_matrix(g2, sparse = FALSE)
  random_modules[k,] <- c(k, module_count_compiled(g3))
}

write.csv(random_modules, "Benguela_LRA.csv")

## Part 4 - Nested Hierarchical I Randomization
## ----------------------------------------------------------------------

Silwood_int <- int_matrix
random_modules <- matrix(nrow = 5000, ncol = 14)
names(random_modules) <- c("Iteration", "Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link")
for (i in 1:5000){
  print(i)
  int_matrix <- Nested_Hierarchical_1_compiled(Silwood_int)
  random_modules[i,] <- c(i, module_count_compiled(int_matrix))
  
}
random_modules <- as.data.frame(random_modules)

write.csv(random_modules, "Benguela_NH1.csv")

## Part 4 - Nested Hierarchical I Randomization
## ----------------------------------------------------------------------

Silwood_int <- int_matrix
random_modules <- matrix(nrow = 5000, ncol = 14)
names(random_modules) <- c("Iteration", "Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link")
for (i in 1:5000){
  print(i)
  int_matrix <- Nested_Hierarchy_2_Compiled(Silwood_int)
  random_modules[i,] <- c(i, module_count_compiled(int_matrix))
  
}
random_modules <- as.data.frame(random_modules)

write.csv(random_modules, "Benguela_NH2.csv")


## Part 6 - Sample Probability Calculation
## ---------------------------------------

Sil_cann <- sum(random_modules$Cannibalism>Observed_Modules$Cannibalism[1])/5000*100  

names(Observed_Modules) <- c("Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link", "Food_Web")
Observed_Modules$z <- Observed_Modules$Cannibalism^2 + Observed_Modules$Mutual_Predation^2 + Observed_Modules$C_Omnivory^2 + Observed_Modules$T_Omnivory^2 + Observed_Modules$Multi_R_Omnivory^2 + Observed_Modules$Apparent_Comp^2 + Observed_Modules$Exploitative_Comp^2 + Observed_Modules$Tri_Trophic_FC^2 + Observed_Modules$Tri_Trophic_FC^2
Observed_Modules$z_cann <- Observed_Modules$Cannibalism/(sqrt(Observed_Modules$z))


random_modules <- read.csv("Benguela_NH2.csv", header= TRUE)
names(random_modules) <- c("X", "Iter", "Cannibalism", "Mutual_Predation", "C_Omnivory", "T_Omnivory", "Multi_R_Omnivory", "Apparent_Comp", "Exploitative_Comp", "Tri_Trophic_FC", "Cann_Link", "MP_Link", "C_omni_Link", "T_omni_Link", "MR_omni_Link")

random_modules$z <- random_modules$Cannibalism^2 + random_modules$Mutual_Predation^2 + random_modules$C_Omnivory^2 + random_modules$T_Omnivory^2 + random_modules$Multi_R_Omnivory^2 + random_modules$Apparent_Comp^2 + random_modules$Exploitative_Comp^2 + random_modules$Tri_Trophic_FC^2 + random_modules$Tri_Trophic_FC^2


Sil_classic <- sum((random_modules$C_Omnivory/sqrt(random_modules$z))>(Observed_Modules$C_Omnivory[1]/sqrt(Observed_Modules$z[1])))/5000*100  
Sil_mp <- sum((random_modules$Mutual_Predation/sqrt(random_modules$z))>(Observed_Modules$Mutual_Predation[1]/sqrt(Observed_Modules$z[1])))/5000*100  
Sil_trophic <- sum((random_modules$T_Omnivory/sqrt(random_modules$z))>(Observed_Modules$T_Omnivory[1]/sqrt(Observed_Modules$z[1])))/5000*100 
Sil_mr <- sum((random_modules$Multi_R_Omnivory/sqrt(random_modules$z))>(Observed_Modules$Multi_R_Omnivory[1]/sqrt(Observed_Modules$z[1])))/5000*100 
Sil_cann <- sum((random_modules$Cannibalism/sqrt(random_modules$z))>(Observed_Modules$Cannibalism[1]/sqrt(Observed_Modules$z[1])))/5000*100  


